/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * dwaite - 20001006
 * Merged all ProtocolHandler code in from sax, as this code will most 
 * probably never be swapped out (a DOM-based implementation would warrant 
 * a substantially different base)
 */

package org.jabber.jabberbeans;

import org.jabber.jabberbeans.sax.*;
import org.jabber.jabberbeans.util.JID;
import java.io.*;
import java.net.*;
import java.util.Enumeration;
import java.util.Vector;

/**
 * ConnectionBean is the main bean for JabberBeans.
 * <p>
 * A ConnectionBean models the connection to the server, and all other beans 
 * rely on the ConnectionBean for server communication.
 * <p>
 * Internally, a ConnectionBean is just a "standard" interface to the backend
 * (by default, Jabberbeans-SAX) which handles the server communication - you
 * simply create objects and push them through 'send' to get to the server,
 * and subscribe to the right interfaces  to 'receive' new objects as they 
 * come in.
 * <p>
 * All the other beans (MessengerBean, IQBean, and RosterBean so far) are
 * considered optional. Internally, they just use the ConnectionBean to 
 * send data, and listen to the events being broadcasted by the 
 * ConnectionBean to filter their 'received' data (which they then 
 * rebroadcast to their respective listeners.)
 * <p>
 * @see MessengerBean
 * @see IQBean
 * @see RosterBean
 * 
 * @see org.jabber.jabberbeans.sax
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: shawnxb $
 * @version $Revision: 1.36 $
 */
public class ConnectionBean implements Serializable, PacketListenerRegistrar
{
    /** holds the current connection status. (e.g. connecting)*/
    private ConnectionEvent.EState status;

    /** list of implementing interfaces which are listening for connection
     *  change events. */
    private final Vector connectionListeners = new Vector();

    /** list of implementing interfaces which are listening for packet
     *  broadcast events. */
    private final Vector packetListeners = new Vector();

    /** Socket, if one is used for the connection. */
    private Socket socket = null;

    /** InputStream handling Thread. This class is responsible for parsing
     *  of external data (via a SAX-enabled parser), and also broadcasts
     *  received packet events. */
    private InputStreamHandler input = null;

    /** OutputStream handling Thread. This class is responsible for sending
     * packets (providing a more asynchronous sending interface), and
     * also broadcasts sent packet events. */
    private OutputStreamHandler output = null;

    /** the default port for jabber servers, 5222 */
    public static final int DEFAULT_PORT = 5222;
		
		/** the default period of time before a connection attempt times out **/
		/** the default time is 10000 ms or 10 seconds **/
		public static final int CONNECTION_TIMEOUT = 10000;
		
		String name = null;

    /** Default namespace for XMLStream */
    private static final String DEFAULT_XMLSTREAM_NS = "jabber:client";

    /** storage for SessionID */
    private String sessionID;
    
    /** used to disable auto-sending XML stream tag */
    private boolean DisableAutoHeader;
		
		
		public class XMLStreamMonitor	implements PacketListener
		{
			public void sentPacket(PacketEvent packetEvent){}
			public void sendFailed(PacketEvent packetEvent){}

			public synchronized void receivedPacket(PacketEvent packetEvent)
			{
				//return if this is not an XML Stream Header
				if (!(packetEvent.getPacket() instanceof XMLStreamHeader))
					return;

				XMLStreamHeader header = (XMLStreamHeader)packetEvent.getPacket();
				sessionID=header.getIdentifier();
				this.notify();
			}
		}

    public class OutputStreamInterface
    {
			public void sent(Packet p)
			{ConnectionBean.this.sent(p);}

			public void sendFailed(Packet p)
			{ConnectionBean.this.sendFailed(p);}

			public void unexpectedThreadDeath(Exception e)
			{ConnectionBean.this.onOutputDeath(e);}
    }

    public class InputStreamInterface
    {
			public void received(Packet p)
			{ConnectionBean.this.received(p);}

			public void unexpectedThreadDeath(Exception e)
			{ConnectionBean.this.onInputDeath(e);}
    }

    public ConnectionBean()
    {
			status = ConnectionEvent.STATE_DISCONNECTED;
			output= new OutputStreamHandler(new OutputStreamInterface());
			input = new InputStreamHandler(new InputStreamInterface());
			sessionID = null;
			DisableAutoHeader=false;
    }

    public void setParser(String s)
    {input.setParser(s);}

    public String getParser()
    {return input.getParser();}

    /**
     * <code>connect</code> attempts to establish a connection to the
     * default port on the specified server.
     *
     * @param host <code>InetAddress</code> of the server
     * @exception java.io.IOException socket error in backend
     */
    public void connect(InetAddress host) throws IOException
    {
			setConnecting();
			connect(host,host.getHostName(),DEFAULT_PORT);
    }
    /**
     * <code>connect</code> attempts to establish a server connection to the
     * default port. This version allows you to specify a string, if the
     * host name the Jabber server expects is different than the one you are
     * connecting with (e.g. for applets).
     *
     * @param host <code>InetAddress</code> of the server
     * @param name <code>String</code> holding the name to send to the Jabber
     *             server on connection.
     * @exception java.io.IOException socket error in backend
     */
    public void connect(InetAddress host, String name) throws IOException
    {
			setConnecting();
			connect(host,name,DEFAULT_PORT);
    }
		
    /**
     * <code>connect</code> attempts to establish a server connection to a
     * specified port. This implementation takes a hostname and port.
     *
     * @param host <code>InetAddress</code> of the server
     * @param port an <code>int</code> specifying the port number
     * @exception java.io.IOException socket error in backend
     */
    public synchronized void connect(InetAddress host, int port) throws IOException
    {
			setConnecting();
			connect(host,host.getHostName(),port);
    }

    /**
     * <code>connect</code> attempts to establish a server connection to a
     * specified port. This version allows you to specify a string, if the
     * host name the Jabber server expects is different than the one you are
     * connecting with (e.g. for applets).
     *
     * @param host <code>InetAddress</code> of the server
     * @param name <code>String</code> holding the name to send to the Jabber
     *             server on connection.
     * @param port an <code>int</code> specifying the port number
     * @exception java.io.IOException socket error in backend
     */
    public synchronized void connect(InetAddress host, String name, int port)	throws IOException
    {
			this.name = name;
			setConnecting();
			Socket s = new Socket(host, port);
			connect(s);
    }
		

    /** 
     * Begin communication with a Jabber server. This method assumes other code
     * responsible for setting up the connection as far as sending the
     * XML Stream header.
     *
     * @param s <code>Socket</code> connecting us with the Jabber server.
     */
    public synchronized void connect(Socket s) throws IOException
    {
			if (socket != null)
				disconnect();
			setConnecting();
			setSocket(s);
			connect(s.getInputStream(),s.getOutputStream());
    }
		public synchronized void connect(Socket s, InetAddress host) throws IOException
    {
			if (socket != null)
				disconnect();
			setConnecting();
			setSocket(s);
			connect(s.getInputStream(),s.getOutputStream());
    }
		/**
     * Begin communication with a Jabber server. This method assumes other code
     * responsible for setting up the connection as far as sending the
     * XML Stream header.
     *
     * @param is <code>InputStream</code> for incoming data from the Jabber
     *           server
     * @param os <code>OutputStream</code> for sending data out to the
     *           Jabber server.
     */
    public synchronized void connect(InputStream is,OutputStream os) throws IOException
    {
			setConnecting();
			
			output.setOutputStream(os);
			output.start();
			
			input.setInputStream(is);
			input.start();
			
			
			if(!DisableAutoHeader) 
			{ 
				XMLStreamMonitor xmlstreammonitor = new	ConnectionBean.XMLStreamMonitor(); 
				addPacketListener(xmlstreammonitor); 
				XMLStreamHeaderBuilder xsbuilder = new XMLStreamHeaderBuilder();
				if(name != null)
					xsbuilder.setToAddress(new JID(this.name));
				else
					xsbuilder.setToAddress(new JID(socket.getInetAddress().getHostName()));

				xsbuilder.setXMLNS(DEFAULT_XMLSTREAM_NS); 

				synchronized(xmlstreammonitor) 
				{
					send(xsbuilder.build());
					try
					{xmlstreammonitor.wait(CONNECTION_TIMEOUT);}
					catch(InterruptedException e){e.printStackTrace();}
				}
				delPacketListener(xmlstreammonitor);
			}
			if(sessionID != null)
				setConnectionState(ConnectionEvent.STATE_CONNECTED, ConnectionEvent.REASON_CLIENT_INITIATED);	
			else
			{
				setConnectionState(ConnectionEvent.STATE_DISCONNECTED, ConnectionEvent.REASON_SERVER_INITIATED);
				//this.notify();
				throw new ConnectException("Session ID NULL.");
			}
		}

    private void setConnecting()
    {
			if (status==ConnectionEvent.STATE_DISCONNECTED)
				setConnectionState(ConnectionEvent.STATE_CONNECTING, ConnectionEvent.REASON_CLIENT_INITIATED);
    }

    private synchronized void setSocket(Socket s)
    {socket=s;}
		
		public final InetAddress getAddress()
		{return socket.getInetAddress();}

		/**
     * <code>disconnect</code> correctly closes down the connection to the
     * Jabber server in the backend.
     */
    public synchronized void disconnect()
    {disconnected(ConnectionEvent.REASON_CLIENT_INITIATED);}

    private synchronized void disconnected(ConnectionEvent.EReason reason)
    {
			if (socket!=null)
			{
				try
				{
					input.shutdown();
					output.shutdown();
					socket.close(); 
					try
					{input.join(10000);}
					catch (InterruptedException e){e.printStackTrace();}
				}
				catch (IOException e){}
				socket=null;
			}
	
			sessionID=null;
			setConnectionState(ConnectionEvent.STATE_DISCONNECTED, reason);
    }

    /**
     * <code>addConnectionListener</code> adds a 
     * <code>ConnectionListener</code> to the list of Listeners to be called
     * when the connection state changes.
     *
     * @param l a <code>ConnectionListener</code>. Cannot add the same listener
     * twice
     */
    public final synchronized void addConnectionListener(ConnectionListener l)
    {
			if(!connectionListeners.contains(l))
    	  connectionListeners.addElement(l);
    }

    /**
     * <code>delConnectionListener</code> removes a previously added
     * <code>ConnectionListener</code> from the list of Listeners to be called
     * when a connection state change event happens.
     *
     * @param l a <code>ConnectionListener</code> value
     */
    public final synchronized void delConnectionListener(ConnectionListener l)
    {
			if(packetListeners.contains(l))
				connectionListeners.removeElement(l);
    }

    /**
     * <code>addPacketListener</code> adds a <code>PacketListener</code> to the
     * list of Listeners to be called when a packet is received or successfully
     * sent out.
     *
     * @param l a <code>PacketListener</code> value
     */
    public final synchronized void addPacketListener(PacketListener l)
    {
			if(!packetListeners.contains(l))
				packetListeners.addElement(l);
    }

    /**
     * <code>delPacketListener</code> removes a 
     * <code>PacketListener</code> to the list of Listeners to be called when 
     * a packet is received or successfully sent out.
     *
     * @param l a <code>PacketListener</code> value
     */
    public final synchronized void delPacketListener(PacketListener l)
    {
			if(packetListeners.contains(l))
				packetListeners.removeElement(l);
    }

    /**
     * <code>send</code> is the method used to send all packets to the server.
     * A <code>Packet</code> is a code element that knows how to explain itself
     * in XML. The <code>Packet</code> is asynchronously sent to the server,
     * with registered <code>ConnectionListener</code>s receiving an event on
     * successful sending.
     *
     * @param packet a <code>Packet</code> being sent to the server
     */
    public void send(Packet packet)
    {output.send(packet);}
    
    /**
     * <code>received</code> is the method used to indicate that a new packet
     * has arrived. This method is <b><i><u>ONLY</u></i></b> meant to be used
     * by the InputStreamHandler thread. Please avoid the urge to use it.
     * It will probably become non-accessable in the near future, consider
     * this a warning.
     *
     * @param packet a <code>Packet</code> arriving from the server.
     */
    public void received(Packet packet)
    {fireReceivedPacket(packet);}

    /**
     * <code>sent</code> is the method used to indicate that a new packet
     * has been sent. This method is <b><i><u>ONLY</u></i></b> meant to be used
     * by the OutputStreamHandler thread. 
     *
     * @param packet a <code>Packet</code> arriving from the server.
     */
    private void sent(Packet packet)
    {fireSentPacket(packet);}

    /**
     * <code>sendFailed</code> is the method used to indicate that a packet
     * failed to send. This method is <b><i><u>ONLY</u></i></b> meant to be
     * used by the OutputStreamHandler thread. 
     *
     * @param packet a <code>Packet</code> arriving from the server.
     */
    private void sendFailed(Packet packet)
    {fireUnsentPacket(packet);}

    /**
     * Internal method which sends out packet events on received data. Called
     * on the InputStreamHandler thread.
     *
     * @param packet the packet we are sending out to all subscribed listeners
     */
    protected final void fireReceivedPacket(Packet packet)
    {
		 	try
			{
				// create a PacketEvent for the broadcast.
				PacketEvent pe=new PacketEvent(this,packet);

				// clone the listeners vector - this is unfortunately the only
				// way to properly allow for random adds/removes. It also allows
				// this method to not be synchronized (hopefully)
				Vector broadcast=(Vector)packetListeners.clone();

		 	  // broadcast to all listeners
				for (Enumeration e=broadcast.elements();e.hasMoreElements();)
				{
					try
					{
							((PacketListener)e.nextElement()).receivedPacket(pe);
					}
					catch (Throwable e1)
					{e1.printStackTrace();}
				}
			}
			catch (Throwable e)
			{e.printStackTrace();}
    }

    /**
     * Internal method which sends out packet events on sent data. Called
     * on the OutputStreamHandler thread.
     *
     * @param packet the packet we are sending out to all subscribed listeners
     */
    protected final void fireSentPacket(Packet packet)
    {
			try
			{
				// create a PacketEvent for the broadcast.
				PacketEvent pe=new PacketEvent(this,packet);

				// clone the listeners vector - this is unfortunately the only way
				// to properly allow for random adds/removes. It also allows this
				// method to not be synchronized (hopefully)
				Vector broadcast=(Vector)packetListeners.clone();

				 // broadcast to all listeners
				for(Enumeration e=broadcast.elements();e.hasMoreElements();)
				{
					try
					{((PacketListener)e.nextElement()).sentPacket(pe);}
					catch (Throwable e1)
					{e1.printStackTrace();}
				}
			}
			catch (Throwable e)
			{e.printStackTrace();}
    }

    /**
     * Internal method which sends out packet events on failed send data. Called
     * on the OutputStreamHandler thread.
     *
     * @param packet the packet we are sending out to all subscribed listeners
     */
    protected final void fireUnsentPacket(Packet packet)
    {
			try
			{
				// create a PacketEvent for the broadcast.
				PacketEvent pe=new PacketEvent(this,packet);

				// clone the listeners vector - this is unfortunately the only way
				// to properly allow for random adds/removes. It also allows this
				// method to not be synchronized (hopefully)
				Vector broadcast=(Vector)packetListeners.clone();

				// broadcast to all listeners
				for(Enumeration e=broadcast.elements();e.hasMoreElements();)
				{
					try
					{((PacketListener)e.nextElement()).sendFailed(pe);}
					catch (Throwable e1)
					{e1.printStackTrace();}
				}
			}
			catch (Throwable e)
			{e.printStackTrace();}
    }

    /**
     * handler code for output stream death.
     */
    public void onOutputDeath(Exception e)
    {
			disconnected(ConnectionEvent.REASON_IO_ERROR);
			// throw new RuntimeException("Death of output thread: " + e.toString());
    }

    /**
     * handler code for input stream death.
     */
    public void onInputDeath(Exception e)
    {
			disconnected(ConnectionEvent.REASON_IO_ERROR);
			// throw new RuntimeException("Death of input thread: " + e.toString());
    }

    /**
     * returns the current connection state (disconnected, connecting, 
     * connected).
     */
    public final ConnectionEvent.EState getConnectionState()
    {return status;}

    /**
     * internal method which allows you to set the connection state,
     * then broadcast the change to all listening classes.
     *
     * @param state new state to enter, based on possibilities from
     *              <code>ConnectionEvent.EState</code>
     * @param reason the reason we are changing states
     *
     * @see ConnectionEvent.EState
     */
    protected final void setConnectionState(ConnectionEvent.EState state, ConnectionEvent.EReason reason)
    {
			ConnectionEvent ce= new ConnectionEvent
					(this, state, getConnectionState(), reason);
			this.status = state;

			//we clone the vector so that we do not need to worry about listeners
			//unsubscribing from events
			Vector broadcast=(Vector)connectionListeners.clone();
			Enumeration e=broadcast.elements();

			//now to broadcast the events
			while (e.hasMoreElements())
			{
				try
				{((ConnectionListener)e.nextElement()).connectionChanged(ce);}
				catch (Throwable e1)
				{e1.printStackTrace();}
			}
    }

    public String getSessionID()
    {return sessionID;}

    public void disableStreamHeader()
    {DisableAutoHeader=true;}
}
