/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.io.Serializable;
import java.util.EventObject;

/**
 * A <code>ConnectionEvent</code> is an event occuring on a connection (from
 * the ConnectionBean) describing a change in the connection state, being 
 * offline, online, connecting, or an error condition (connectFailed).
 *
 * The main purpose for a ConnectionEvent currently is to mark the 'source'
 * of the event, so that the same event handler code can handle multiple
 * connections.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.9 $
 */
public class ConnectionEvent
    extends EventObject
{
    public final static EState STATE_UNKNOWN =
	new EState (EState.NOT_SET);
    public final static EState STATE_DISCONNECTED =
	new EState (EState.DISCONNECTED);
    public final static EState STATE_CONNECTING =
	new EState (EState.CONNECTING);
    public final static EState STATE_CONNECTED =
	new EState (EState.CONNECTED);

    public final static EReason REASON_UNKNOWN =
	new EReason (EReason.NOT_SET);
    public final static EReason REASON_CLIENT_INITIATED =
	new EReason (EReason.CLIENT_INITIATED);
    public final static EReason REASON_SERVER_INITIATED =
	new EReason (EReason.SERVER_INITIATED);
    public final static EReason REASON_IO_ERROR =
	new EReason (EReason.IO_ERROR);


    
    /**
     * EState is an enumerator class; it provides an immutable
     * reference on the current state of the connection to a jabber server.
     *
     * @author  David Waite <a href="mailto:dwaite@jabber.com">
     *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
     */
    public static final class EState
    {
	/* EState enumeration values */
	public final static int NOT_SET      =-1;
	public final static int DISCONNECTED = 0;
	public final static int CONNECTING   = 1;
	public final static int CONNECTED    = 2;


	/** stores the value of the enumeration - initially 'not set' */
	private int value = NOT_SET;
	
	/**
	 * Creates a new immutable <code>EState</code> instance.
	 *
	 * @param newVal an <code>int</code> value, meant to be one of the
	 * connection states defined in this class
	 * @exception IllegalArgumentException if an illegal state is declared
	 */
        protected EState(int newVal)
	    throws IllegalArgumentException
	{
	    if ((newVal<-1)||(newVal>2))
		throw new 
		    IllegalArgumentException("Invalid Enumeration Value");
	    value=newVal;
	}
	
	/**
	 * converts the value to a string for display.
	 *
	 * @return a <code>String</code> holding the enumeration type name
	 */
	public String toString()
	{
	    switch(value)
	    {
	    case NOT_SET:
		return "NOT_SET";
	    case DISCONNECTED:
		return "DISCONNECTED";
	    case CONNECTING:
		return "CONNECTING";
	    case CONNECTED:
		return "CONNECTED";
	    default:
		return "Internal Error";
	    }
	}
	
	/**
	 * <code>getValue</code> returns the value of the enumeration.
	 *
	 * @return an <code>int</code> value corresponding to the enumeration
	 */
	public int getValue()
	{ return value; }
    }

    public final static class EReason
    {
	/* EReason enumeration values */
	public static final int NOT_SET = -1;
	public static final int CLIENT_INITIATED = 0;
	public static final int SERVER_INITIATED = 1;
	public static final int IO_ERROR = 2;

	private int value;

	protected EReason(int reason)
	    throws IllegalArgumentException
	{
	    if ( (reason<-1) || (reason>2) )
		throw new 
		    IllegalArgumentException("Invalid Enumeration Value");
	    value = reason;
	}
	/**
	 * <code>getValue</code> returns the value of the enumeration.
	 *
	 * @return an <code>int</code> value corresponding to the enumeration
	 */
	public int getValue()
	{ return value; }

	/**
	 * converts the value to a string for display.
	 *
	 * @return a <code>String</code> holding the enumeration type name
	 */
	public String toString()
	{
	    switch(value)
	    {
	    case NOT_SET:
		return "NOT_SET";
	    case CLIENT_INITIATED:
		return "CLIENT_INITIATED";
	    case SERVER_INITIATED:
		return "SERVER_INITIATED";
	    case IO_ERROR:
		return "IO_ERROR";
	    default:
		return "Internal Error";
	    }
	}
    }


    private EState state, oldState;
    private EReason reason;
    
    /**
     * Creates a new <code>ConnectionEvent</code> instance. This should
     * only really be called by ConnectionBean.
     *
     * @param source an <code>Object</code> value
     * @param state an <code>Estate</code> value
     * @param oldState an <code>EState</code> value
     * @param reason an <code>Estate</code> value
     */
    public ConnectionEvent(Object source, EState state, 
			   EState oldState, EReason reason)
    {
	super(source);
	this.state=state;
	this.oldState=oldState;
	this.reason=reason;
    }

    /**
     * <code>getState</code> returns the state change which prompted this
     * event.
     *
     * @return an <code>EState</code> value
     */
    public EState getState()
    { return state; }

    /**
     * <code>getOldState</code> returns the state which we were in before this
     * event
     *
     * @return an <code>EState</code> value
     */
    public EState getOldState()
    { return oldState; }

    /**
     * <code>getReason</code> returns the reason the state changed, indicating
     * why the source sent this event (connection lost,
     * client requested disconnect, server requested disconnect)
     *
     * @return an <code>EReason</code> value
     */
    public EReason getReason()
    { return reason; }
}
