/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;
import org.jabber.jabberbeans.Extension.Extension;
import org.jabber.jabberbeans.util.JID;

/**
 * A <code>ContentPacket</code> is derived from a <code>Packet</code>,a single
 * unit of information to be sent (or received) from the server. The three 
 * base types of packets currently 'known' are IQ, Presence and Message.<p>
 *
 * The <code>ContentPacket</code> is an abstract class that combines elements
 * of these three types together, such as destination and origination (to and
 * from) addresses, the xml identifier, packet type, etc. This creates a
 * significant improvement on the amount of duplicated code between these three
 * elements.
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.21 $
 */
public abstract class ContentPacket extends XMLData implements Packet,Extension,Serializable
{
	/** The address that this packet is being sent to - the destination for
	 * outgoing packets, you (the recipient) for incoming packets. */
	protected JID toAddress;
	/** The address that this packet is coming from - your address (or blank)
	 * for outgoing packets, the originator's address for incoming packets. */
	protected JID fromAddress;

	/** An identifier that is echoed in data replies/errors of this packet.
	 * Identifiers are meant to be unique per XML stream, and not duplicated.*/
	protected String identifier;

	/** A packet subtype. For instance, a message can be a normal message, a
	 * message in a one-on-one chat, a groupchat message or a headline. */
	protected String type;

	/** Error code on event of an error (type=error). This is supposed to 
	 * always hold the numerical value of the error code (which are very
	 * close to the HTTP error codes).*/
	protected String errorCode;

	/** Extended description of the error. */
	protected String errorText;

	/** A list of extensions attached to this object. Note that some derived
	 * classes will support only one extension at a time (e.g. InfoQuery)
	 */
	protected Vector extensions;

	/**
	 * This constructor builds the packet portion of packet-derived objects. 
	 * Note that due to different requirements of packet-derived objects, 
	 * these values are not checked. 
	 *
	 * Only derived types are real Jabber data elements, so you cannot directly
	 * create a ContentPacket object. Create a Message, Presence, or InfoQuery
	 * instead.
	 *
	 * @param packet a <code>PacketBuilder</code> which holds the value of the
	 * packet parameters
	 * @return the beginnings of a <code>Packet</code> object.
	 */
	protected ContentPacket(PacketBuilder packet)
	{
		toAddress=packet.getToAddress();
		fromAddress=packet.getFromAddress();
		type=packet.getType();
		
		identifier=packet.getIdentifier();
		
		/* note: we always use an identifier- if one is not specified by
		 * the client, we insert our own. */
		if(identifier==null)
			identifier=IdentifierCounter.getNewIdentifier();
		else if(identifier.equals("none"))
			//they specified they wanted no identifier - set to null.
			identifier=null;

		errorCode=packet.getErrorCode();
		errorText=packet.getErrorText();
		extensions=packet.getExtensionVector();
	}

	/**
	 * <code>getToAddress</code> returns the address of the recipiant of this
	 * message. Can be blank (null)
	 *
	 * @return <code>JID</code> who this packet was addressed to.
	 */
	public JID getToAddress()
	{return toAddress;}

	/**
	 * <code>getFromAddress</code> returns the address of the sender of this
	 * message. Can be blank (null)
	 *
	 * @return <code>JID</code> this packet originated from.
	 */
	public JID getFromAddress()
	{return fromAddress;}

	/**
	 * <code>getIdentifier</code> returns a string which uniquely identifies 
	 * this packet in the XML stream. Any reply (such as an IQ type=result or
	 * an error message) will carry this same identifier. Identifiers are not
	 * meant to be duplicated in a stream.
	 *
	 * @return a <code>String</code> value
	 */
	public String getIdentifier()
	{return identifier;}

	/**
	 * <code>getType</code> returns the subtype of this packet. Its context
	 * and correctness is determined by the final packet type (message, iq,etc)
	 * according to the jabber protocol specifications on <a 
	 * href="http://protocol.jabber.org">protocol.jabber.org</a>.
	 *
	 * @return <code>String</code> holding the type
	 */
	public String getType()
	{return type;}

	/**
	 * <code>getErrorCode</code> gets the HTTP-style error code for this packet
	 * . All packets can potentially return errors. Error information without
	 * the packet having subtype type="error" is considered invalid.
	 *
	 * @return a <code>String</code> holding the error code
	 */
	public String getErrorCode()
	{return errorCode;}

	/**
	 * <code>getErrorText</code> gets the error text, a fuller textual
	 * description of the problem encountered. Error information without the
	 * packet having subtype type="error" is considered invalid.
	 *
	 * @return a <code>String</code> value
	 */
	public String getErrorText()
	{return errorText;}

	/**
	 * <code>Extensions</code> returns an enumeration of extensions, if
	 * any
	 *
	 * @return an <code>Enumeration</code> value, or null if no extensions
	 */
	public Enumeration Extensions()
	{
		if(extensions == null)
			return null;
		return extensions.elements();
	}


	/**
	 * <code>appendBaseAttribs</code> is a helper funnction which adds the 
	 * base attributes to a StringBuffer which is being used to be return 
	 * a XML representation of the packet..
	 *
	 * @param sb a <code>StringBuffer</code> value
	 */
	protected void appendBaseAttribs(StringBuffer sb)
	{
		appendAttrib(sb,"to",toAddress);
		appendAttrib(sb,"from",fromAddress);
		appendAttrib(sb,"id", identifier);
		appendAttrib(sb,"type",type);
	}

	/**
	 * <code>appendAnyError</code> will add the error information,if any exists
	 * for this packet.
	 *
	 * @param sb StringBuffer which we are injecting data into
	 */
	protected void appendAnyError(StringBuffer sb)
	{
		if(!"error".equals(type) || errorCode == null || errorCode.length() <= 0)
			return;
		
		sb.append("<error");
		sb.append(" code=\"" + errorCode         + "\"");
		
		if(errorText == null)
		{
			sb.append("/>");
			return;
		}
		sb.append(">");
		escapeString(sb,errorText);
		sb.append("</error>");
  }
}
