/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import org.jabber.jabberbeans.util.JID;
import org.jabber.jabberbeans.*;

/**
 * <code>Agent</code> is a class representing a single Agent/Transport/Gateway.
 * It works both as the <agent ...> tag inside of a jabber:iq:agents namespace,
 * and alone as a jabber:iq:agent namespace.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.13 $
 */
public class Agent
    extends XMLData
    implements QueryExtension
{
    /** 
     * <code>jid</code> holds the address of the agent, if this is a child 
     * of the agents namespace. If it is an agent namespace, then the jid is
     * already accessable, it is the address you sent the request to, or the
     * address the packet came back from.
     */
    private JID jid;

    /** <code>name</code> of the transport. */
    private String name;

    /** short textual <code>description</code> of the transport. */
    private String description;

    /** <code>service</code> name. */
    private String service;

    /** <code>transport</code> holds the formal 'username' of the transport. */
    private String transport;

    /** <code>register</code> is true if you must register to play. */
    private boolean register;

    /** <code>search</code> is true if the agent is searchable. */
    private boolean search;

    /** <code>IQAgent</code> differentiates whether this is a sub-tag of the
     * agents namespace, or our own agent namespace. True means we are our own
     * namespace, jabber:iq:agent.
     */
    private boolean IQAgent;

    /**
     * Creates a new <code>Agent</code> instance, based on AgentBuilder
     *
     * @param builder an <code>AgentBuilder</code> value
     * @exception InstantiationException if invalid
     */
    public Agent(AgentBuilder builder)
        throws InstantiationException
    {
        IQAgent=builder.isIQAgent();
        jid=builder.getJID();
        if (IQAgent)
        {
            if (jid!=null)
                throw new InstantiationException
                    ("JID not allowed for iq:agent");
        }
        else
        {
            if (jid==null) 
                throw new InstantiationException
                    ("JID required for agent");
        }
        name=builder.getName();
        description=builder.getDescription();
        service=builder.getService();
        register=builder.isRegister();
        search=builder.isSearchable();
        transport=builder.getTransport();
    }

    /**
     * <code>getName</code> returns the agent name.
     *
     * @return a <code>String</code> value
     */
    public String getName()
    { return name; }

    /**
     * <code>getJID</code> returns the address which this agent is at.
     * Note that this is not valid for a jabber:iq:agent, only if this
     * is an element of an jabber:iq:agents list.
     */
    public JID getJID()
    { return jid; }

    /**
     * <code>getDescription</code> returns the agent short description.
     *
     * @return a <code>String</code> value
     */
    public String getDescription()
    { return description; }
    
    /**
     * <code>getService</code> returns the service name.
     *
     * @return a <code>String</code> value
     */
    public String getService()
    { return service; }

    /**
     * <code>getTransport</code> returns the transport user-name.
     *
     * @return a <code>String</code> value
     */
    public String getTransport()
    { return transport; }

    /**
     * <code>isRegister</code> returns true if you must register to use this
     * transport
     *
     * @return a <code>boolean</code> value
     */
    public boolean isRegister()
    { return register; }

    /**
     * <code>isSearchable</code> returns true if the agent allows searches
     * for users.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isSearchable()
    { return search; }

    /**
     * <code>isIQAgent</code> returns true if we are a jabber:iq:agent
     * extension
     *
     * @return a <code>boolean</code> value
     */
    public boolean isIQAgent()
    { return IQAgent; }

    /**
     * <code>appendItem</code> converts this packet to XML, either an <agent>
     * block in a jabber:iq:agents extension, or its own jabber:iq:agent
     * extension. It then appends it to a StringBuffer
     *
     * @return a <code>StringBuffer</code>
     */
    public void appendItem(StringBuffer retval)
    {
        retval.append('<');
        if (isIQAgent())
            retval.append("query xmlns=\"jabber:iq:agent\"");
        else
            retval.append("agent");

	appendAttrib(retval, "jid", jid);

        retval.append('>');

        //other children
        appendChild(retval, "name", name);
        appendChild(retval, "description", description);
        appendChild(retval, "service", service);
        appendChild(retval, "register", register);
        appendChild(retval, "search", search);
        appendChild(retval, "transport", transport);
        if (isIQAgent())
            retval.append("</query>");
        else
            retval.append("</agent>");
    }
}
