/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import org.jabber.jabberbeans.util.JID;

/**
 * <code>AgentBuilder</code> is a class which constructs single 
 * Agent/Transport/Gateway entries, either as <agent> tags within a 
 * jabber:iq:agents namespace, or as its own jabber:iq:agent namespace.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.6 $
 */
public class AgentBuilder
{
    /** 
     * <code>jid</code> holds the address of the agent, if this is a child 
     * of the agents namespace. If it is an agent namespace, then the jid is
     * already accessable, it is the address you sent the request to, or the
     * address the packet came back from.
     */
    private JID jid;

    /** <code>name</code> of the Transport. */
    private String name;

    /** short textual <code>description</code> of the transport. */
    private String description;

    /** <code>service</code> name. */
    private String service;

    /** <code>transport</code> holds the formal 'username' of the transport */
    private String transport;

    /** <code>register</code> is true if you must register to play. */
    private boolean register;

    /** <code>search</code> is true if the agent is searchable. */
    private boolean search;

    /** <code>IQAgent</code> differentiates whether this is a sub-tag of the
     * agents namespace, or our own agent namespace. True means we are our own
     * namespace, jabber:iq:agent.
     */
    private boolean IQAgent;

    /** Creates a new <code>AgentBuilder</code> instance. */
    public AgentBuilder()
    { reset(); }
 
    /** 
     * <code>reset</code> restores defaults to all values, so that this 
     * builder can be reused.
     */
    public void reset()
    {
	jid=null;
	name=description=service=transport=null;
	register=search=false;
	IQAgent=false;
    }

    /**
     * <code>getJID</code> returns the jid
     *
     * @return a <code>JID</code> value
     */
    public JID getJID()
    { return jid; }

    /**
     * <code>setJID</code> sets the jid, or clears it if null is passed in
     *
     * @param val a <code>JID</code> value, or null to clear
     */
    public void setJID(JID val)
    { jid=val; }

    /**
     * <code>getName</code> returns the name of the agent, or null if none.
     *
     * @return a <code>String</code> value
     */
    public String getName()
    { return name; }

    /**
     * <code>setName</code> sets the name of the transport
     *
     * @param val a <code>String</code> value, or null to clear.
     */
    public void setName(String val)
    { name=val; }

    /**
     * <code>getDescription</code> returns the short description, or null if
     * none.
     *
     * @return a <code>String</code> value
     */
    public String getDescription()
    { return description; }

    /**
     * <code>setDescription</code> sets the short agent description.
     *
     * @param val a <code>String</code> value, or null to clear.
     */
    public void setDescription(String val)
    { description=val; }

    /**
     * <code>getService</code> returns the service name of the agent, or null
     * if none.
     *
     * @return a <code>String</code> value
     */
    public String getService()
    { return service; }

    /**
     * <code>setService</code> sets the service name of the agent.
     *
     * @param val a <code>String</code> value, or null to clear.
     */
    public void setService(String val)
    { service=val; }

    /**
     * <code>isRegister</code> returns true if you need to register to use this
     * agent.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isRegister()
    { return register; }

    /**
     * <code>setRegister</code> sets whether or not this agent needs 
     * registration
     *
     * @param val a <code>boolean</code> value
     */
    public void setRegister(boolean val)
    { register=val; }

    /**
     * <code>isSearchable</code> indicates if this agent supports searching.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isSearchable()
    { return search; }

    /**
     * <code>setSearchable</code> sets whether or not this agent supports
     * searching.
     *
     * @param val a <code>boolean</code> value
     */
    public void setSearchable(boolean val)
    { search=val; }

    /**
     * <code>getTransport</code> holds the context-specific name of the user
     * in regards to this Agent/Transport/Gateway
     *
     * @return a <code>String</code> value, or null if none.
     */
    public String getTransport()
    { return transport; }

    /**
     * <code>setTransport</code> sets the field name of the user with respect
     * to the Agent/Transport/Gateway.
     *
     * @param val a <code>String</code> value, or null to clear.
     */
    public void setTransport(String val)
    { transport=val; }

    /**
     * <code>isIQAgent</code> returns true if we are to construct our own
     * agent namespace object, and false if this is just a member of
     * jabber:iq:agents.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isIQAgent()
    { return IQAgent; }

    /**
     * <code>setIQAgent</code> sets whether this is an agent namespace, or a
     * member of jabber:iq:agents.
     *
     * @param val a <code>boolean</code> value
     */
    public void setIQAgent(boolean val)
    { IQAgent=val; }

    /**
     * <code>build</code> an Agent object.
     *
     * @return an <code>Agent</code> value
     * @exception InstantiationException if inadequate data was provided.
     */
    public Agent build()
	throws InstantiationException
    { return new Agent(this); }
}

