package org.jabber.jabberbeans.Extension;


import java.util.*;
import org.jabber.jabberbeans.util.*;
import org.jabber.jabberbeans.*;


/*
 * A <code>BrowseItem</code> Extension object is used to represent a single
 * item in a browse result. A BrowseItem can contain other BrowseItems, and
 * also may contain a list of supported namespaces.
 */

public class BrowseItem extends XMLData implements QueryExtension
{
	/* iq packet or not */
	private boolean iq;
	
	/** category of the browse item */
	private String category;

	/** type of the browse item */
	private String type;

	/** name given to this browse item */
	private String name;

	/** address this browse item refers to */
	private JID jid;

	/* namespaces held by this item */
	private Vector namespaces;
	
	/* child browseitems held by this item */
	private Vector childItems; 

	/** Construct a new <code>BrowseItem</code> object, based on the builder
	 * state.
	 *
	 * @param builder BrowseItemBuilder which holds the state this object
	 *                will be based on
	 */
	public BrowseItem(BrowseItemBuilder builder)
	{
		iq = builder.isIQ();
		category = builder.getCategory();
		type = builder.getType();
		name = builder.getName();
		jid = builder.getJID();
		namespaces = builder.getNameSpaces();
		childItems = builder.getChildren();
	}
	
	/* return the IQ status */
	public final boolean isIQ()
	{return iq;}

	/** return the category of this item (represented by the type attribute) */
	public final String getCategory()
	{return category;}

	/** return the type of this item (represented by the element name) */
	public final String getType()
	{return type;}

	/** return a textual string for displaying this item */
	public final String getName()
	{return name;}

	/** return the address this item refers to */
	public final JID getJID()
	{return jid;}
	
	/* return the namespaces vector itself */
	public Vector getNamespaces()
	{return namespaces;}
	
	/* return the children vector itself */
	public Vector getChildren()
	{return childItems;}

	/* return an enumeration of namespaces as strings, or null if no
	 * namespaces were specified. 
	 */
	public final Enumeration namespaces()
	{
		if (namespaces == null)
			return null; 
		return namespaces.elements();
	}

	/** return an enumeration of children as BrowseItems, or null if no
	 *  children were specified. */ 
	public final Enumeration children()
	{
	 if (childItems == null)
		 return null;
	 return childItems.elements();
	}

	
	/**
	 * <code>getXMLNS</code> is overloaded by derived classes to allow the
	 * toString and appendItem() code to be fixed.
	 */
  protected String getXMLNS()
	{return null;}


	/**
	 * <code>appendItem</code> converts this packet to XML
	 * It then appends it to a StringBuffer.
	 *
	 * @param retval The <code>StringBuffer</code> to append to
	 */
	public void appendItem(StringBuffer retval)
	{
		retval.append('<');
		if(category == null)
			retval.append("query");
		else
			retval.append(category);
		if(isIQ())
			appendAttrib(retval, "xmlns", getXMLNS());
		appendAttrib(retval, "type", getType());
		appendAttrib(retval, "name", getName());
		appendAttrib(retval, "jid", getJID());
		retval.append('>');
		Enumeration e = namespaces.elements();
		while (e.hasMoreElements())
		{
			retval.append("<ns>");
			retval.append(e.nextElement().toString());
			retval.append("</ns>");
		}
		e = childItems.elements();
		while (e.hasMoreElements())
		{
			BrowseItem bi = (BrowseItem) e.nextElement();
			bi.appendItem(retval);
		}
		retval.append("</");
		if(category == null)
			retval.append("query");
		else
			retval.append(category);
		retval.append('>');
	}
}