/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import org.jabber.jabberbeans.*;

/**
 * An <code>IQAuth</code> object represents the jabber:iq:auth
 * namespace. This namespace extension is used to specify the
 * information needed to log into the jabber server.<p>
 *
 * This object can currently use 1 of 3 different strategies to
 * authenticate with the server: Plaintext, Digest, or Zero
 * Knowledge. Plaintext sends the password in plaintext, and is
 * the least secure. Digest sends a SHA1 digest of the password
 * to the server. Zero-Knowledge uses a shared-secret to
 * authenticate the user without sending the password at all. Not
 * all servers support all of these strategies: the client developer
 * should determine which of these methods are supported and then
 * supply the necessary information. If you provide
 * <code>IQAuth</code> with data for more than 1 authentication
 * strategy, it will try to use strategies in this order:
 * <ol>
 * <li>Zero-Knowledge
 * <li>SHA1 Digest
 * <li>Plaintext
 * </ol>
 *
 * @see <a href="http://docs.jabber.org/draft-proto/html/zerok.html">
 *       Zero-Knowledge Specification</a>
 * @see <a href="http://docs.jabber.org/proto/html/jabber%3Aiq%3Aauth.html">
 *       Simple Client Authentication Specification</a>
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.18 $
 */
public class IQAuth
    extends XMLData
    implements QueryExtension
{
    /** a <code>String</code> which represents the user name on the server. */
    private String username;

    /** a <code>String</code> which represents the session resource. */
    private String resource;

    /** a <code>String</code> which holds the password for this user. */
    private String password;

    /** 
     * a <code>String</code> which holds the SHA1 Digest for this user.
     * The digest authentication is a way to log in without sending the
     * password over the wire in plaintext. Both entities are still required
     * to know the plaintext password. The server sends a SessionID as part
     * of the initial &lt;stream:stream&gt; tag. This values is prepended to the
     * user password, then run through a SHA hash and converted to Hex format.
     * Hashes are non-reversable, so the original password is not discernable,
     * but the server can perform the same steps and compare output to log
     * a user in.
     *
     * @see org.jabber.jabberbeans.util.SHA1Helper
     */
    private String SHA1Digest;

    /* hash value for doing zero-k authentication with the server */
    private String ZeroKHash;

    /* value for seeding zero-k authentication */
    private String ZeroKToken;

    /* sequence count for performing zero-k authentication */
    private int sequence;

    /** Creates a new <code>IQAuthExtension</code> instance. This will fail if
     * the builder does not have sufficient or correct information for an IQ
     * Auth snippet.
     *
     * @param builder an <code>IQAuthExtensionBuilder</code> which holds the
     * values we are using.
     *
     * @exception InstantiationException thrown if the builder does not hold
     * sufficient data to create a valid Extension snippet.
     */
    public IQAuth(IQAuthBuilder builder)
        throws InstantiationException
    {
        username=builder.getUsername();
        resource=builder.getResource();

        password=builder.getPassword();
        SHA1Digest=builder.getSHA1Digest();
	ZeroKHash=builder.getZeroKHash();
	ZeroKToken=builder.getZeroKToken();
	sequence=builder.getZeroKSequence();
    }

    /**
     * <code>getUsername</code> returns the value of the username attribute
     *
     * @return a <code>String</code> value
     */
    public String getUsername()
    {
        return username;
    }

    /**
     * <code>getResource</code> returns the value of the resource attribute.
     *
     * @return a <code>String</code> value
     */
    public String getResource()
    {
        return resource;
    }

    /**
     * <code>getPassword</code> returns the value used for the password.
     *
     * @return a <code>String</code> value
     */
    public String getPassword()
    {
        return password;
    }

    /**
     * <code>getSHA1Digest</code> returns the value used for the SHA1 digest.
     *
     * @return a <code>String</code> value
     */
    public String getSHA1Digest()
    {
        return SHA1Digest;
    }

    public String getZeroKHash()
    {
	return ZeroKHash;
    }

    public String getZeroKToken()
    {
	return ZeroKToken;
    }

    public int getZeroKSequence()
    {
	return sequence;
    }

    /**
     * <code>appendItem</code> appends the XML representation of the
     * current packet data to the specified <code>StringBuffer</code>.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public void appendItem(StringBuffer retval)
    {
        retval.append("<query xmlns=\"jabber:iq:auth\">");
	appendChild(retval,"username",username);
	appendChild(retval,"password",password);
	appendChild(retval,"digest",SHA1Digest);
	appendChild(retval,"resource",resource);
	appendChild(retval,"hash",ZeroKHash);
	appendChild(retval,"token",ZeroKToken);
	appendChild(retval,"sequence",
		    ((sequence==-1)?null:new Integer(sequence).toString()));
	
	retval.append("</query>");
    }
}
