/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import java.io.Serializable;
import org.jabber.jabberbeans.*;

/**
 * An <code>OOB</code> object represents Out-of-band referenced data, and
 * represents the core data passed via a jabber:x:oob or jabber:iq:oob 
 * namespace extension. Additional functionality (URL verification, user 
 * download facility, mini-HTTP server) is not provided here.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.7 $
 */
public class OOB
    extends XMLData
    implements MessageExtension,
               QueryExtension,
               PresenceExtension
{
    /**
     * This boolean is set true for jabber:iq:oob, left false for 
     * jabber:x:oob
     */
    private boolean IQ;

    /** String holding the URL of the out-of-band data. */
    private String URL;

    /** String holding and description of the data being referenced. */
    private String description;

    /**
     * Construct a new OOB object based on the builder state
     *
     * @param builder <code>OOBBuilder</code>which holds the values which
     *                this object will be created.
     */
    public OOB(OOBBuilder builder)
    {
	IQ=builder.isIQ();
	URL=builder.getURL();
	description=builder.getDescription();
    }

    /**
     * Determine if this is a jabber:x:oob or jabber:iq:oob extension
     *
     * @return <code>True</code> if this is a jabber:iq:oob extension,
     *         <code>False</code> if this is a jabber:x:oob extension
     */
    public boolean isIQ()
    {
	return IQ;
    }

    /**
     * get the URL to which this OOB is pointing
     *
     * @return <code>String</code> holding the URL this OOB is based around
     */
    public String getURL()
    {
	return URL;
    }

    /**
     * get a Description of the resource which this OOB represents
     *
     * @return <code>String</code>holding a description of the URL.
     */
    public String getDescription()
    {
	return description;
    }

    /**
     * <code>appendItem</code> appends the XML representation of the
     * current packet data to the specified <code>StringBuffer</code>.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public void appendItem(StringBuffer retval)
    {
	if (IQ)
	    retval.append("<query xmlns=\"jabber:iq:oob\">");
	else
	    retval.append("<x xmlns=\"jabber:x:oob\">");

	appendChild(retval,"url",URL);
	appendChild(retval,"desc",description);

	if (IQ)
	    retval.append("</query>");
	else
	    retval.append("</x>");
    }
}
