/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import java.util.Enumeration;
import java.util.Vector;
import org.jabber.jabberbeans.*;

/**
 * A <code>Roster</code> holds the values that would be stored in
 * a jabber:iq:roster or a jabber:x:roster namespace. It is a list of users 
 * <p>
 * a jabber:iq:roster object is requested from the server to get the active
 * user's contact list, sent to the server to modify the contact list, and
 * also received from the server ('pushed') when the contact list changes,
 * by any active resource of the active user.
 * <p>
 * a jabber:x:roster object is sent in messages, to give another user certain
 * contacts in a processable form, so that a specific UI can be shown
 * (and the user doesn't have to enter all these resources by hand)
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.13 $
 */
public class Roster
    extends XMLData
    implements QueryExtension,
               MessageExtension
{
    /** Vector holding the individual roster items */
    private Vector rosterItems=null;

    /**
     * <code>IsIQRoster</code> is true if this is a jabber:iq:roster 
     * extension, false if it is a jabber:x:roster.
     */
    private boolean IsIQRoster;

    /**
     * Creates a new <code>Roster</code> instance.
     *
     * @param builder an <code>RosterBuilder</code> value
     */
    public Roster(RosterBuilder builder)
        throws InstantiationException
    {
        if (builder.getRosterItems()==null)
        {
            this.rosterItems=null;
            return;
        }

        IsIQRoster=builder.isIQRoster();
        this.rosterItems=(Vector)builder.getRosterItems().clone();
    }

    /**
     * <code>items</code> returns an Enumeration which can be
     * used to step through all the available jabber roster items.
     *
     * @return an <code>Enumeration</code>, or null if no items present
     */
    public Enumeration items()
    {
        if (rosterItems==null)
            return null;
        return rosterItems.elements();
    }

    /**
     * <code>isIQRoster</code> returns true if this is a jabber:iq:roster, 
     * false if it is a jabber:x:roster.
     *
     * @return a <code>boolean</code> value
     */
    public boolean isIQRoster()
    {
        return IsIQRoster;
    }

    /**
     * <code>appendItem</code> converts this packet to XML, then appends 
     * it to a StringBuffer
     *
     * @param retval a <code>StringBuffer</code> to append to.
     */
    public void appendItem(StringBuffer retval)
    {   
        if (IsIQRoster)
            retval.append( "<query xmlns=\"jabber:iq:roster\"" );
        else
            retval.append( "<x xmlns=\"jabber:x:roster\"" );

        Enumeration s=items();
        if (s==null)
        {
            retval.append("/>");
            return;
        }
        else retval.append(">");
        
        while (s.hasMoreElements())
        {
            retval.append(s.nextElement());
        }

        if (IsIQRoster)
            retval.append( "</query>" );
        else
            retval.append( "</x>" );
    }
}
