/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import org.jabber.jabberbeans.*;

/**
 * An <code>UpdateInfo</code> is used to relay information about a particular
 * available update.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.9 $
 */
public class UpdateInfo
    extends XMLData
{
    /** 
     * <code>type</code> is the type of update, be it a "release", a "beta", 
     * or a "dev" release.
     */
    private String type; 

    /**
     * <code>version</code> is the version of the software being reported,
     * hopefully the most recent release available.
     */
    private String version;
    
    /**
     * <code>URL</code> holds the unique resource locator value for the
     * client (or user) to get a software update.
     */
    private String URL;
    
    /**
     * <code>priority</code> indicates how severe the priority of the release
     * has been ranked. Options are "optional" or "mandatory"
     */
    private String priority;

    /**
     * Creates an <code>UpdateInfo</code> object.
     */
    public UpdateInfo(UpdateInfoBuilder builder)
    {
	type=builder.getType();
	version=builder.getVersion();
	URL=builder.getURL();
	priority=builder.getPriority();
    }

    /**
     * <code>getType</code> returns the type of this particular update, be it
     * a release, a beta, or a developer's version.
     *
     * @return String holding type value
     */
    public String getType()
    {
	return type;
    }

    /**
     * <code>getVersion</code> returns the version number of the newest
     * release (at least, the newest release that the agent knows about)
     *
     * @return String holding version text
     */
    public String getVersion()
    {
	return version;
    }

    /**
     * <code>getURL</code> returns the internet address of the software,
     * either to be fetched automatically by the client, or manually by the
     * user.
     */
    public String getURL()
    {
	return URL;
    }
    
    /**
     * <code>getPriority</code> returns the priority of the update, whether
     * it be an optional or mandatory upgrade from the previous version.
     */
    public String getPriority()
    {
	return priority;
    }

    /**
     * <code>appendItem</code> appends the XML representation of the
     * current packet data to the specified <code>StringBuffer</code>.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public void appendItem(StringBuffer retval)
    {
	retval.append('<');
	retval.append(type);
	appendAttrib(retval,"priority",priority);
	retval.append(">");

	appendChild(retval,"ver",version);
	appendChild(retval,"url",URL);
	
	retval.append("</");
	retval.append(type);
	retval.append('>');
    }
}
