/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.Extension;

import org.jabber.jabberbeans.util.JID;

/**
 * A <code>XDelay</code> extension handles jabber:x:delay packets. These are
 * used to represent any type of delay in the path of a message or presence
 * packet - for instance, from messages sent offline, or messages sent through
 * transports from a third-party system.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.5 $
 */
public class XDelayBuilder
    implements ExtensionBuilder
{
    /** fromAddress is the party causing the delay */
    private JID fromAddress;

    /** stamp is the time stamp from when the message was delayed */
    private String stamp;

    /** content holds a string describing why the packet was delayed */
    private String content;

    /** construct a new XDelayBuilder object */
    public XDelayBuilder()
    {
	reset();
    }

    /** reset the builder to a default state, for reuse */
    public void reset()
    {
	fromAddress=null;
	stamp=null;
	content=null;
    }

    
    /**
     * get the address of the delaying party
     *
     * @return <code>JID</code> of the party which delayed the message.
     */
    public JID getFromAddress()
    {
	return fromAddress;
    }

    /**
     * set the address of the delaying party
     *
     * @param from <code>JID</code> of the party which delayed the message.
     */
    public void setFromAddress(JID from)
    {
	fromAddress=from;
    }

    /**
     * get the timestamp of when the packet was delayed
     *
     * @return <code>String</code> holding the ISO date stamp
     */
    public String getStamp()
    {
	return stamp;
    }

    /**
     * set the timestamp of when the packet was delayed
     *
     * @param stamp <code>String</code> holding the ISO date stamp
     */
    public void setStamp(String stamp)
    {
	this.stamp=stamp;
    }

    /**
     * get the content describing why the message was delayed
     *
     * @return <code>String</code> holding text content
     */
    public String getContent()
    {
	return content;
    }

    /**
     * set the content describing why the message was delayed
     *
     * @param txt <code>String</code> holding text content
     */
    public void setContent(String txt)
    {
	content=txt;
    }

    /**
     * build a new <code>XDelay Extension</code> based on the current
     * builder state
     *
     * @return <code>XDelay Extension</code>
     * @throws InstantiationException if there is not sufficient information
     *         to construct a full XDelay object.
     */
    public Extension build()
	throws InstantiationException
    {
	return new XDelay(this);
    }
}
