package org.jabber.jabberbeans;


import org.jabber.jabberbeans.*;
import org.jabber.jabberbeans.util.JID;
import org.jabber.jabberbeans.Extension.*;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.*;
import java.io.*;
import java.lang.InstantiationException;


public class GroupChatBean implements Serializable
{
	//The address for this bean
	private JID address = null;
	
	//A way of implementing a debug mode
	private boolean debug = false;
	
	//Create a connection bean.
	private ConnectionBean cb = null;
	
	//Create a list of the presence listeners
	private Vector chatListeners = new Vector();
	
	//Create a list of the user presences with resources
	private Hashtable participantList = new Hashtable();
	
	//Create a presence listener
	private InternalPacketListener listener = null;
	
	//reference to the presence bean that we've blacklisted in
	private PresenceBean pb = null;
	
	
	/*
	 * <code>GroupChatBean</code> is the constructor
	 * 
	 * @param cb <code>ConnectionBean</code> is the connection bean to send on
	 * @param address <code>JID</code> is the address the bean is handling
	 * @param pb <code>PresenceBean</code> is the presence bean we're blacklisting in
	 */
	public GroupChatBean(ConnectionBean cb, JID address, PresenceBean pb)
	{reset(cb, address, pb);}
	
	
	/*
	 * <code>reset</code> Called by the constructor to blacklist the address in the 
	 * presence bean.
	 *
	 * @param address a <code>JID</code> is for setting the address for this bean
	 * @param pb a <code>PresenceBean</code> where we blacklist the address
	 *
	 */
	public void reset(ConnectionBean cb, JID address, PresenceBean pb)
	{
		try{cb.delPacketListener(listener);}
		catch(Exception e){e.printStackTrace();}
		
		listener = new InternalPacketListener();
		cb.addPacketListener(listener);

		this.cb = cb;
		this.address = address;
		this.pb = pb;
		
		//Clear the list of people in the room
		participantList.clear();

		//Enable the blacklist in the presence bean
		pb.setBlacklistEnable(true);
		//Add the room address to the list
		pb.addToBlacklist(new JID(address.getUsername(), address.getServer(), null));
		//Send an available presence to initiate entrance
		sendPresence(null, "available", "available");
	}
	
	
	/*
	 * <code>shutdown</code> shuts down the bean and removes the blacklisted item
	 *
	 */
	public void shutdown()
	{
		pb.removeFromBlacklist(address);
		sendPresence("unavailable", "unavailable", "unavailable");
	}
	
	
	/*
	 * <code>setAddress</code> sets the address
	 *
	 * @param str is a <code>String</code> and is the addres of the room we're handling
	 */
	public void setAddress(JID jid)
	{
		if(jid != null)
			address = jid;
		
		sendPresence(null, "available", "available");
	}
	
	
	/*
	 * <code>getAddress</code> returns the address
	 *
	 * @return <code>JID</code>
	 */
	public JID getAddress()
	{return address;}
	
	
	/*
	 * <code>setNickName</code> sets the nickname to use in the room
	 *
	 * @param <code>String</code> nickName to change to
	 */
	public void setNickName(String nickName)
	{
		if(nickName == null || "".equals(nickName))
			return;
		
		address = new JID(address.getUsername(), address.getServer(), nickName);
		sendPresence(null, "available", "available");		
	}
	
	
	/*
	 * <code>getNickName</code> returns the nickName we're using in the room
	 *
	 * @return <code>String</code>
	 */
	public String getNickName()
	{return address.getResource();}
		
	
	/*
	 * <code>sendPresence</code> is used for sending prsence to the room however you like
	 * You must always send the state and status
	 *
	 * @param type is a <code>String</code> that's for the type of the presence packet, available, unavailable, etc.
	 * @param state is a <code>String</code> what are state is, available, away, etc.
	 * @param status is a <code>String</code> the status message we're using 
	 *
	 * @return <code>boolean</code> indicating if the send was completed w/o errors
	 */
	public boolean sendPresence(String type, String state, String status)
	{
		if(state == null || status == null)
			return false;
		
		try
		{
			PresenceBuilder builder = new PresenceBuilder();

			builder.reset();
			if(type != null)
				builder.setType(type);
			builder.setStateShow(state);
			builder.setStatus(status);
			builder.setToAddress(address);
			
			cb.send(builder.build());
			return true;
		}
		catch(Exception e){e.printStackTrace(); return false;}
	}
	
	
	/*
	 * <code>sendMessage</code> accepts a subject and a body and returns true on a successful send
	 * You must always send a body, subject however is optional
	 *
	 * @param subject <code>String</code> is the subject of the message
	 * @param body <code>String</code> is the body of the message
	 *
	 * @return <code>boolean</code> indicating if the send was completed w/o errors
	 */
	public boolean sendMessage(String subject, String body)
	{
		//We never accept a blank body
		if(body == null)
			return false;
		
		try
		{
			MessageBuilder builder = new MessageBuilder();
			
			builder.reset();
			builder.setToAddress(new JID(address.getUsername(), address.getServer(), null));
			builder.setType("groupchat");
			if(subject != null)
				builder.setSubject(subject);
			builder.setBody(body);

			cb.send(builder.build());
			return true;
		}
		catch(Exception e){e.printStackTrace(); return false;}
	}
	
	
	//packet listener
	class InternalPacketListener implements PacketListener
	{
		public void receivedPacket(PacketEvent pe)
		{
			try
			{
				if(debug)
					System.err.println(pe.getPacket());
				
				//Test for presence packets
				if(pe.getPacket() instanceof Presence)
				{
					Presence temp = (Presence)pe.getPacket();
					if(temp.getFromAddress().toSimpleString().equals(address.toSimpleString()))
						PresenceHandler(temp);
					return;
				}
				
				//Test for message packets
				if(pe.getPacket() instanceof Message)
				{					
					Message temp = (Message)pe.getPacket();
					if(temp.getFromAddress().toSimpleString().equals(address.toSimpleString()))
						MessageHandler((Message)pe.getPacket());
					return;
				}				
			}
			catch(Exception e){e.printStackTrace();}
		}
		
		//never used
		public void sentPacket(PacketEvent pe){}	
		public void sendFailed(PacketEvent pe){}		
	}
	
	
	public void PresenceHandler(Presence p)
	{		
		String type = p.getType();
		JID addy = p.getFromAddress();
		String state = p.getStateShow();
		String status = p.getStatus();
		
		//JID short_address = JID.fromString(addy.getResource());
		JID full_address = addy;
		JID room_address = JID.fromString(addy.toSimpleString());
		boolean room_contained = participantList.containsKey(room_address);				
		
		
		if(type == (null) || "available".equals(type))
		{//user has come online or changed state or status
			
			//Only allow an addition if the list doesn't contain the element
			//Otherwise do a compare before adding or removing
			if(!room_contained)
			{//user has come online for the first time.
			
				//notification
				if(debug)
					System.err.println("Adding user the first time.");
					
				//create the one time usernode.  This contains individual resources
				PresenceUserNode node = new PresenceUserNode(addy, state, status);
				
				if(debug)
					System.err.println("Node: " + node);
					
				//place node in tree
				participantList.put(room_address, node);

				//fire the changed presence handler with a copy of the current list of
				//online users
				fireChangedPresence(participantList, p, node, "online");

				//notification
				if(debug)
					System.err.println("Fired changed presence event.");

			}
			else
			{//a new resource has come online, or an existing one has bee modified

				//check for a different resource, or change the state info
				PresenceUserNode node = (PresenceUserNode)participantList.get(room_address);

				if(node.containsResource(full_address))
				{//the node contains the resource, so we change the state, and status info

					//notification
					if(debug)
						System.err.println(full_address + " adjusted for state info.");

					//set the resources state
					node.setState(addy, state);

					//set the resources status
					node.setStatus(addy, status);

					//fire the changed presence handler with a copy of the current list of
					//online users
					fireChangedPresence(participantList, p, node, "status");

				}
				else
				{//the node does NOT contain the resource, so we add it

					//notification
					if(debug)
						System.err.println(addy + " added as a resource to the user node.");

					//add a resource to the user's node
					node.addResource(addy, state, status);

					if(debug)
						System.err.println("Adding resource " + addy + ".\nResources: " + node.getNumResources());

					//fire the changed presence handler with a copy of the current list of
					//online users
					fireChangedPresence(participantList, p, node, "online");

				}


				//notification
				if(debug)
					System.err.println("Fired changed presence event.");

			}				
		}
		else if("unavailable".equals(type)) 
		{//user resource has gone offline

			if(room_contained)
			{//Only allow a remove if the list actually contains the element

				//get the room node so we can test for multiple users
				PresenceUserNode node = (PresenceUserNode)participantList.get(room_address);

				if(node.getNumResources() <= 1)
				{//we have a single resource, so we can remove the entire reference

					//remove the user's node from the running list of online users
					participantList.remove(room_address);

					if(debug)
						System.err.println("Removing room " + room_address);

				}
				else
				{//we have more than one resource, so we must remove just it.

					//notification
					if(debug)
						System.err.println("Removing resource " + full_address + ".\nResources: " + node.getNumResources());

					//remove the resource from the user's node
					node.removeResource(full_address);

					//notification
					if(debug)
						System.err.println("Removing resource " + full_address + ".\nResources: " + node.getNumResources());

				}
			//fire the changed presence handler with a copy of the current list of
			//online users
			fireChangedPresence(participantList, p, null, "offline");
			}

		}
		else
		{//we've received some sort of error packet from the user or their server

			if(debug)
				System.err.println("Error presence packet recieved");

			//fire the error event handler
			fireError(p);

		}
	}
	
	public void MessageHandler(Message m)
	{
		//Fire the event handler
		if("groupchat".equals(m.getType()))
			fireMessageReceived(m);
	}

	
	
	//Set the connection bean
	/*
	 * <code>setConnBean</code> This will set the connection bean that we read packets from.
	 *
	 * @param connbean a <code>ConnectionBean</code> used for reading packets off of.
	 *
	public void setConnBean(ConnectionBean connbean)
	{
		this.cb = connbean;
		listener = new InternalPacketListener();
		cb.addPacketListener(listener);
	}
	 */
	
	
	/*
	 * <code>getConnBean</code> This will return the connection bean we're using.
	 *
	 * @return <code>ConnectionBean</code>
	 */
	public ConnectionBean getConnBean()
	{return this.cb;}
	
	
	/*
	 * <code>getParticipantList</code> returns the participant list for the room
	 *
	 * @return <code>Hashtable</code>
	 */
	public Hashtable getParticipantList()
	{return this.participantList;}
	
	
	/*
	 * <code>getDebug</code> returns the debug status of the bean
	 *
	 * @return <code>boolean</code>
	 */
	public boolean getDebug()
	{return this.debug;}
	
	
	/*
	 * <code>setDebug</code> sets the debug status of the bean
	 *
	 * @param b <code>boolean</code> takes a boolean and sets the debug state
	 */
	public void setDebug(boolean b)
	{this.debug = b;}
	
	
	public String toString()
	{
		String temp = new String();
		Collection collection = participantList.values();
		Iterator iterator = collection.iterator();
		
		while(iterator.hasNext())
		{
			PresenceUserNode node = (PresenceUserNode)iterator.next();			
			temp += node.toString();
		}
		return temp;
	}
	
	//Changed Presence event
	public void fireChangedPresence(Hashtable t, Presence p, PresenceUserNode n, String s)
	{
		try
		{
			Vector broadcast = (Vector)chatListeners.clone();
			for(Enumeration e = broadcast.elements(); e.hasMoreElements();)
			((GroupChatListener)e.nextElement()).ChangedPresence(t, p, n, s);
		}
		catch(Exception e){e.printStackTrace();}
	}
	
	
	
	//Error event
	public void fireError(Presence p)
	{
		try
		{
			Vector broadcast = (Vector)chatListeners.clone();
			for(Enumeration e = broadcast.elements(); e.hasMoreElements();)
			((GroupChatListener)e.nextElement()).Error(p);
		}
		catch(Exception e){e.printStackTrace();}
	}
	
	
	//Message event
	public void fireMessageReceived(Message m)
	{
		try
		{
			Vector broadcast = (Vector)chatListeners.clone();
			for(Enumeration e = broadcast.elements(); e.hasMoreElements();)
			((GroupChatListener)e.nextElement()).ReceivedMessage(m);
		}
		catch(Exception e){e.printStackTrace();}
	}
	
	
	/*
	 * <code>addGroupChatListener</code add the <code>GroupChatListener</code> specified
	 *
	 * @param <code>GroupChatListener</code> l is the listener to add
	 */
	public synchronized void addGroupChatListener(GroupChatListener l)
	{
		if(!chatListeners.contains(l))
			chatListeners.addElement(l);
	}
	
	
	/*
	 * <code>delGroupChatListener</code removes the <code>GroupChatListener</code> specified
	 *
	 * @param <code>GroupChatListener</code> l is the listener to remove
	 */
	public synchronized void delGroupChatListener(GroupChatListener l)
	{ 
		if(chatListeners.contains(l))
			chatListeners.removeElement(l); 
	}
}