/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * IQBean is a bean used to utilize the 'iq' functionality of the jabber 
 * protocol. IQ is used for setting and reading values, including user 
 * information (for logging on).
 * <p>
 * IQBean, like most of the other beans in jabberbeans, relies on 
 * ConnectionBean to provide the actual network socket to the server. IQBean
 * will not operate until a connection is set.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: dwaite $
 * @version $Revision: 1.12 $
 */

public class IQBean 
    implements Serializable,
	       PacketListenerRegistrar
{
    /** holds a reference to the main connection bean. */
    private ConnectionBean connection=null;
    
    /** rebroadcaster, part of our filter for connectionbean methods */
    private IQListener rebroadcaster=null;
   
    /**
     * <code>IQListener</code> is a PacketListener interface, it listens
     * to packets, filters for IQ packets, and then calls fire methods to
     * tell the main class that a packet has been 'found'.
     */
    class IQListener
        extends PacketRebroadcaster
    {
        /**
         * waits for packet, then fires main interface when detected
         * @param p <code>PacketEvent</code> received
         */
        public void receivedPacket(PacketEvent p)
        {
            if (p.getPacket() instanceof InfoQuery)
                fireReceived(p);
        }

        /**
         * waits for packet, then fires main interface when detected
         * @param p <code>PacketEvent</code> sent
         */
        public void sentPacket(PacketEvent p)
        {
            if (p.getPacket() instanceof InfoQuery)
                fireSent(p);
        }

        /**
         * waits for packet, then fires main interface when detected
         * @param p <code>PacketEvent</code> sent
         */
        public void sendFailed(PacketEvent p)
        {
            if (p.getPacket() instanceof InfoQuery)
                fireSendFailed(p);
        }
    }

    /**
     * Creates a new <code>IQBean</code> instance.
     *
     */
    public IQBean()
    { rebroadcaster=new IQListener(); }
    
    /**
     * Creates a new <code>IQBean</code> instance, and registers it with an
     * existing connection
     *
     * @param connection <code>ConnectionBean</code> we will monitor
     */
    public IQBean(ConnectionBean connection)
    {
        this();
        setConnection(connection);
    }

    /**
     * Send out a packet through the ConnectionBean.
     *
     * @param packet an <code>InfoQuery</code> packet
     */
    public void send(InfoQuery packet)
    {
        if (connection==null) 
            throw new RuntimeException
                ("IQBean not connected to a Connection object");
        connection.send(packet);
    }

    /**
     * sets the connection bean required for the IQBean to operate
     *
     * @param connection a <code>ConnectionBean</code> value which maps to
     * a server connection.
     */
    public void setConnection(ConnectionBean connection)
    {
        if (this.connection!=null)
            throw new RuntimeException
                ("Cannot change connection of an active IQ Bean");

        this.connection=connection;
        rebroadcaster.registerListener(connection);
    }

    /**
     * returns the connection bean in use, or null if none
     *
     * @return <code>ConnectionBean</code> value
     */
    public ConnectionBean getConnection()
    { return connection; }

    /**
     * <code>addPacketListener</code> registers a packet listener with our 
     * filtered packet rebroadcaster.
     *
     * @param l a <code>PacketListener</code> value
     */
    public final void addPacketListener(PacketListener l)
    { rebroadcaster.addPacketListener(l); }

    /**
     * <code>delPacketListener</code> unregisters a packet listener with our
     * filtered packet rebroadcaster.
     *
     * @param l a <code>PacketListener</code> value
     */
    public final void delPacketListener(PacketListener l)
    { rebroadcaster.delPacketListener(l); }
}
