/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.util.Enumeration;
import java.io.Serializable;

/**
 * This is the data type that holds the data of a message.
 * from the jabber documentation page <a 
 * href="http://docs.jabber.org/documents/developers/protocol/messages.html">
 * here </a> we get the message format (sorry, not in DTD format):<p>
 * <p>
 * &lt;message to="*" from="*" id="*" 
 * type="error|normal|chat|groupchat"&gt;<p>
 * <p><i>The following metadata</i><br>
 *      &lt;subject&gt;subject text&lt;/subject&gt;<br>
 *      &lt;thread&gt;thread id&lt;/thread&gt;<p>
 * <p><i> and the following content types:</i><p>
 *      &lt;body&gt;main text of message&lt;/body&gt; <p>
 *      &lt;ext xmlns="jabber:x:*"&gt;&lt;/ext&gt; <p>
 * &lt;/message&gt;
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.18 $
 */
public final class Message
    extends ContentPacket
    implements Serializable
{
    /** Subject string */
    private String subject;

    /** Thread string */
    private String thread;
    /** Body string */
    private String body;

    /**
     * Builds a message, based on the data in the MessageBuilder.
     *
     * @param builder 'MessageBuilder' to feed in.
     * @exception InstantiationException thrown if not enough/malformed data 
     * was in the MessageBuilder
     */
    public Message(MessageBuilder builder)
        throws InstantiationException
    {
        super(builder);
              
        if ((type == null)||(type.equals("normal")))
            type = null;
        else
	    type.intern();

        subject = builder.getSubject();
        thread  = builder.getThread();
        body    = builder.getBody();

	if (subject!=null) subject.intern();
	if (thread!=null) thread.intern();
	if (body!=null) body.intern();
    }
    /**
     * <code>getSubject</code> returns the text within the subject element, if
     * any
     *
     * @return a <code>String</code> value, or null if no subject
     */
    public final String getSubject()
    { return subject; }

    /**
     * <code>getThread</code> returns the text within the thread element, if
     * any
     *
     * @return a <code>String</code> value, or null if no thread
     */
    public final String getThread()
    { return thread; }

    /**
     * <code>getBody</code> returns the text within the body element, if
     * any
     *
     * @return a <code>String</code> value, or null if no body
     */
    public final String getBody()
    { return body; }

    /**
     * <code>appendItem</code> appends the XML representation of the
     * current packet data to the specified <code>StringBuffer</code>.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public final void appendItem(StringBuffer retval)
    {
        //first we contruct the '<message ...>' tag
	retval.append("<message");

        //append the contentPacket attributes.
        appendBaseAttribs(retval);
        retval.append(">");

        //tack on error info, if any
        appendAnyError(retval);

        appendChild(retval,"subject",subject);
        appendChild(retval,"thread",thread);
        appendChild(retval,"body",body);

        //then, all 'ext' parts. note that we require a full tag
        //here, while all other parts just require the data

        Enumeration extensions=Extensions();
        if (extensions!=null)
            while (extensions.hasMoreElements())
                retval.append(extensions.nextElement());


        //finally, the closing
        retval.append("</message>");
    }
}