/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import org.jabber.jabberbeans.Extension.*;
import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;

/**
 * This class is used to build the complex 'Message' object. After entering 
 * all the appropriate commands, you call build(), which returns a message 
 * object.
 *
 * Unlike most Builder Patterns, this is not used to create different types of
 * objects. However, Message objects are meant to be immutable, and
 * the complex creation (variable number and types of arguments) made this
 * class much simpler.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.comgt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.12 $
 */
public class MessageBuilder
    extends PacketBuilder
    implements Serializable
{
    /** thread identifier. indicates the message is still in the same 'thread',
     * i.e that it is the same discussion. The client returns the thread
     * unchanged if it is a reply to a message. */
    private String thread=null;

    /** subject line of message */
    private String subject=null;

    /** body of message. */
    private String body=null;

    public MessageBuilder()
    { reset(); };

    /** Reset prepares the message builder class to build a new message. */
    public void reset()
    {
        resetBase();
        subject = body = thread = null;
    }

    /** Build a new Message object based on the current builder state.
     *
     * @return Message created with the properties
     * @exception InstantiationException thrown if a valid message cannot 
     * be constructed from the current parameters.
     */
    public Packet build() 
        throws InstantiationException
    { return new Message(this); }

    /**
     * set the discussion thread that the message belongs to.
     *
     * @param thread a value of type 'String'
     */
    public void setThread(String thread)
    { this.thread=thread; }

    /**
     * get the current discussion thread this message is being created with.
     *
     * @return a value of type 'String'
     */
    public String getThread()
    { return thread;}

    /**
     * set the subject of the message
     *
     * @param subject a value of type 'String'
     */
    public void setSubject(String subject)
    { this.subject=subject;}

    /**
     * get the subject of the message
     *
     * @return a value of type 'String'
     */
    public String getSubject()
    { return subject; }

    /**
     * set the message body (the meat of the message)
     *
     * @param body a value of type 'String'
     */
    public void setBody(String body)
    { this.body=body; }

    /**
     * get the message body
     *
     * @return a value of type 'String'
     */
    public String getBody()
    { return body; }
}
