/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * MessengerBean is used to send and receive messages over
 * a Connection. Connections are established by the 
 * ConnectionBean and required to use the MessengerBean.
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: dwaite $
 * @version $Revision: 1.13 $
 */
public class MessengerBean
    implements Serializable,
               PacketListenerRegistrar
{
    /** Base connection bean, required for communications */
    private ConnectionBean connection=null;

    /** rebroadcaster, for filtering the packet listener from the connection
     * bean down and provide another subsribable subset. */
    private PacketRebroadcaster rebroadcaster=null;

    /**
     * class <code>MessageListener</code> finished implementing the 
     * PacketRebroadcaster code. This class filters out all but messages.
     */
    class MessageListener
        extends PacketRebroadcaster
    {
        /**
         * <code>receivedPacket</code> filters out incoming packets
         *
         * @param p a <code>PacketEvent</code> value
         */
        public void receivedPacket(PacketEvent p)
        {
            if ((p.getPacket() instanceof Message)&&
		(p.getSource()==connection))
		
                fireReceived(p);
        }

        /**
         * <code>sentPacket</code> filters out sent packet events
         *
         * @param p a <code>PacketEvent</code> value
         */
        public void sentPacket(PacketEvent p)
        {
            if ((p.getPacket() instanceof Message)&&
		(p.getSource()==connection))

                fireSent(p);
        }

	/**
	 *
	 * <code>sendFailed</code> filters out failed packet events
	 *
	 * @param p a <code>PacketEvent</code> value
	 */
	public void sendFailed(PacketEvent p)
	{
            if ((p.getPacket() instanceof Message)&&
		(p.getSource()==connection))

		fireSendFailed(p);
	}

    }
    
    /** Creates a new <code>MessengerBean</code> instance. */
    public MessengerBean()
    { rebroadcaster=new MessageListener(); }

    /**
     * Creates a new <code>MessengerBean</code> instance, and registers it
     * with a connection
     *
     * @param connection a <code>ConnectionBean</code> to use for this
     * MessengerBean
     */
    public MessengerBean(ConnectionBean connection)
    {
        this();
        setConnection(connection);
    }
    
    /**
     * <code>setConnection</code> sets the ConnectionBean after creation
     *
     * @param connection a <code>ConnectionBean</code> to use for this
     * MessengerBean
     */
    public void setConnection(ConnectionBean connection)
    {
        this.connection=connection;
        rebroadcaster.registerListener(connection);
    }

    /**
     * <code>getConnection</code> returns the connectionbean this object
     * is using.
     *
     * @return <code>ConnectionBean</code> this class is registered with
     */
    public ConnectionBean getConnection()
    { return connection; }

    /**
     * <code>send</code> a Message packet
     *
     * @param message a <code>Message</code> value
     */
    public void send(Message message)
    {
        if (connection==null)
            throw new RuntimeException
                ("MessengerBean not connected to a ConnectionBean object");
        connection.send(message);
    }

    /**
     * <code>addPacketListener</code> registers a new packet listener with this
     * bean.
     *
     * @param l a <code>PacketListener</code> to register
     */
    public synchronized void addPacketListener(PacketListener l)
    { rebroadcaster.addPacketListener(l); }

    /**
     * <code>delPacketListener</code> unregisters a packet listener with this
     * bean.
     *
     * @param l a <code>PacketListener</code> to unregister
     */
    public synchronized void delPacketListener(PacketListener l)
    { rebroadcaster.delPacketListener(l); }
}
