/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import org.jabber.jabberbeans.util.JID;
import org.jabber.jabberbeans.Extension.*;
import java.util.Vector;
import java.io.Serializable;

/**
 * A <code>PacketBuilder</code> is the base class for the builder of all 
 * the different Builder classes for Packet objects. Packet-derived objects
 * require builders since they are immutable.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.19 $
 */
public abstract class PacketBuilder
    implements Serializable
{
    /** JID for recipient of the packet */
    protected JID toAddress;

    /** JID for the originator of the packet */
    protected JID fromAddress;

    /** unique identifier for the packet */
    protected String identifier;

    /** Subtype of the packet(defined based on the final packet type) */
    protected String type;

    /** Error code of a packet of type="error" */
    protected String errorCode;
    /** Error Textual Description of a packet of type="error" */
    protected String errorText;

    /** extensions associated with this packet. */
    protected Vector extensions;

    /**
     * <code>reset</code> is a method implemented by children to reset their
     * data.
     */
    abstract public void reset();

    /**
     * <code>resetBase</code> resets all information in this base class.
     *
     */
    protected final void resetBase()
    {
        toAddress=fromAddress=null;
        identifier=type=errorCode=errorText=null;
        extensions=new Vector();
    }

    /** get the current recipient
     *
     * @return a value of type 'JID'
     */
    public JID getToAddress()
    { return toAddress; }

    /** set the current recipient
     *
     * @param user a value of type 'JID'
     */
    public void setToAddress(JID to)
    { toAddress=to; }

    /** get the sender address
     *
     * @return a value of type 'JID'.
     */
    public JID getFromAddress()
    { return fromAddress; }

    /** set the sender address
     *
     * @param user a value of type 'JID'.
     */
    public void setFromAddress(JID from)
    { fromAddress=from; }

    /**
     * get the message identifier. Note that a message identifier is
     * created on the message if one isn't passed in here - read the
     * built message if you would like to retrieve this (final) identifier.
     *
     * @return a value of type 'String'
     */
    public String getIdentifier()
    { return identifier; }

    /**
     * Set the message identifier. Note that if you do not set this,
     * the message will generate one on creation.
     *
     * @param id string value of the identifier.
     */
    public void setIdentifier(String id)
    { identifier=id; }

    /**
     * get the message type. Will return null for default (being type=normal)
     *
     * @return a value of type 'String'
     * @see <a href="http://protocol.jabber.org/">Jabber Protocol Site</a>
     */
    public String getType()
    { return type; }
    /**
     * set the message type
     *
     * @param type a value of type 'String'
     * @see <a href="http://protocol.jabber.org/">Jabber Protocol Site</a>
     */
    public void setType(String type)
    { this.type=type; }

    /**
     * get the error code number
     *
     * @return a value of type String
     */
    public String getErrorCode()
    { return errorCode; }

    /**
     * set the error type number
     *
     * @param type new error type
     */
    public void setErrorCode(String errorCode)
    { this.errorCode=errorCode; }

    /**
     * get the error textual description
     *
     * @return a value of type 'String'
     */
    public String getErrorText()
    { return errorText; }

    /**
     * set the error textual description
     *
     * @param text new error description
     */
    public void setErrorText(String errText)
    { errorText=errText; }      

    /**
     * Add an extension to this object
     *
     * @param e an <code>Extension</code> value
     */
    public void addExtension(Extension e)
    {
        if (!extensions.contains(e))
            extensions.addElement(e);
    }

    /**
     * Remove an extension from this object
     *
     * @param e an <code>Extension</code> value
     */
    public void delExtension(Extension e)
    {
        if (extensions.contains(e))
            extensions.removeElement(e);
    }

    /**
     * Returns a copy of the Vector of Extension objects
     *
     * @return an <code>Vector</code> value, or null if no extensions
     */
    public Vector getExtensionVector()
    {
        if (extensions==null)
            return null;
        return (Vector)extensions.clone();
    }

    /**
     * <code>build</code> will create a packet based on the information in
     * the subclassed builder
     *
     * @return a <code>Packet</code> value on success
     * @exception InstantiationException if insufficient information was 
     * provided to construct a packet.
     */
    abstract public Packet build()
        throws InstantiationException;
}
