/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;

/**
 * <code>PacketRebroadcaster</code> is a class used by the optional beans to
 * allow for subscriptions to 'filtered' interfaces. By using this class and
 * overriding the PacketListener members, you can filter for a packet that 
 * meets specific requirements.
 * <p>
 * Known issues:
 * <ul><li>an attempt to modify the list (the Vector) while enumerating
 * the list will cause the attempt to go to the next listener to fail. I
 * (mass) am evaluating other ways to handle this; so far the only solution
 * I have seen is to clone the vector before enumerating. I would like to know
 * if there are other solutions before I do this.
 *
 * </li><li>the fire events need to catch *all* exceptions and do something 
 * with them. Currently they fall back into the parser, which causes
 * everything to stop.
 * 
 * </li></ul>
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.15 $
 */
public abstract class PacketRebroadcaster
    implements PacketListener,
               Serializable,
	       PacketListenerRegistrar
{
    /** Vector holding people who are listening for packet events */
    private Vector listeners=new Vector();

    /**
     * <code>registerListener</code> registers the listener with the connection
     * bean, so that it will receive events.
     *
     * @param cb a <code>ConnectionBean</code> value
     */
    public void registerListener(ConnectionBean cb)
    { cb.addPacketListener(this); }

    /**
     * <code>unregisterListener</code> unregisters the listener with the 
     * connection bean, so that it will no longer receive events.
     *
     * @param cb a <code>ConnectionBean</code> value
     */
    public void unregisterListener(ConnectionBean cb)
    { cb.delPacketListener(this); }

    /**
     * PacketListener element
     *
     * @param pe a <code>PacketEvent</code> value
     */
    public abstract void receivedPacket(PacketEvent pe);

    /**
     * PacketListener element
     *
     * @param pe a <code>PacketEvent</code> value
     */
    public abstract void sentPacket(PacketEvent pe);

    /**
     * PacketListener element
     *
     * @param pe a <code>PacketEvent</code> value
     */
    public abstract void sendFailed(PacketEvent pe);

    /**
     * <code>addPacketListener</code> registers a packet listener with our 
     * filtered packet rebroadcaster.
     *
     * @param l a <code>PacketListener</code> value
     */
    public final void addPacketListener(PacketListener l)
    { listeners.addElement(l); }

    /**
     * <code>delPacketListener</code> unregisters a packet listener with our
     * filtered packet rebroadcaster.
     *
     * @param l a <code>PacketListener</code> value
     */
    public final void delPacketListener(PacketListener l)
    {
        if (listeners.contains(l))
            listeners.removeElement(l);
    }

    /**
     * <code>fireReceived</code> is called by the PacketListener-derived
     * ReceivedPacket. It sends the same packet out to everyone registered to
     * this rebroadcaster.
     *
     * @param p a <code>PacketEvent</code> value
     */
    protected final void fireReceived(PacketEvent p)
    {
	Vector broadcast=(Vector)listeners.clone();
	Enumeration e = broadcast.elements();
	while (e.hasMoreElements())
	{
	    try
	    {
		((PacketListener)e.nextElement()).receivedPacket(p);
	    }
	    catch (Throwable e1)
	    { }
	} 
    }

    /**
     * <code>fireSent</code> is called by the PacketListener-derived
     * SentPacket. It sends the same packet out to everyone registered to
     * this rebroadcaster.
     *
     * @param p a <code>PacketEvent</code> value
     */
    protected final void fireSent(PacketEvent p)
    {
	Vector broadcast=(Vector)listeners.clone();
	Enumeration e = broadcast.elements();
	while (e.hasMoreElements())
	{
	    try
	    {
		((PacketListener)e.nextElement()).sentPacket(p);
	    }
	    catch (Throwable e1)
	    { }
	}
    }

    /**
     * <code>fireSendFailed</code> is called by the PacketListener-derived
     * SentPacket. It sends the same packet out to everyone registered to
     * this rebroadcaster.
     *
     * @param p a <code>PacketEvent</code> value
     */
    protected final void fireSendFailed(PacketEvent p)
    {
	Vector broadcast=(Vector)listeners.clone();
	Enumeration e = broadcast.elements();
	while (e.hasMoreElements())
	{
	    try
	    {
		    ((PacketListener)e.nextElement()).sendFailed(p);
	    }
	    catch (Throwable e1)
	    { }
	}
    }
}
