/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */


package org.jabber.jabberbeans;


import org.jabber.jabberbeans.Extension.*;
import java.util.Enumeration;
import java.io.Serializable;
import java.util.*;


/**
 * This class is an abstraction of presence packets - both incoming and 
 * outgoing.
 *
 * Incoming presence packets are notifications on a resource - they are
 * sent by the remote client's server when that client has a status change.
 *
 * Outgoing presence packets are of two types. You send out your presence by
 * sending a packet to the server with your new status. The second type is a
 * subscribe request, where you request a remote user add you to their
 * presence notifications.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.19 $
 */


public final class Presence extends ContentPacket implements Serializable
{
	/* status string. */
	private String status;

	/* priority - higher is better. */
	private int priority;

	/* stateshow - the icon 'state' of the user */
	private String stateShow;

	/* Vector of valid types 
	static Vector validTypes = new Vector();
	static
	{
		validTypes.addElement(null);
		validTypes.addElement("unavailable");
		validTypes.addElement("suscribe");
		validTypes.addElement("subscribed");
		validTypes.addElement("unsubscribe");
		validTypes.addElement("unsubscribed");
		validTypes.addElement("invisible");
		validTypes.addElement("probe");
	}
	*/
	
	
	/**
	 * Creates a new <code>Presence</code> instance.
	 *
	 * @param builder a <code>PresenceBuilder</code> to get parameters from
	 * @exception InstantiationException if there is not enough data to create
	 * a valid packet.
	 */
	public Presence(PresenceBuilder builder) throws InstantiationException
	{
		super((PacketBuilder)builder);
		
		if("available".equals(type))
			type = null;
		
		
		//This is commented out because it causes random disconnects for an unknown reason
		//If the type is not specified in the valid list above, abort
		//if(!validTypes.contains(type))
		//	throw new InstantiationException("Invalid type specified.  See http://docs.jabber.org/jpg/html/main.html#REFPRESENCE for a valid list");
		
		
		status=builder.getStatus();
		stateShow=builder.getStateShow();
		priority=builder.getPriority();
		
		
		if(priority < 0 && priority != -66666)
			throw new InstantiationException("Invalid (negative) priority on presence information");
	}
	
	
	/**
	 * <code>getStatus</code> returns the status value, if any
	 *
	 * @return a <code>String</code> value of status, or null if none
	 */
	public final String getStatus()
	{ return status; }
	
	
	/**
	 * <code>getPriority</code> returns the priority level being set
	 *
	 * @return an <code>int</code> value
	 */
	public final int getPriority()
	{ return priority; }
	
	
	/**
	 * <code>getStateShow</code> returns the State of a user
	 *
	 * @return a <code>String</code> value showing state
	 */
	public final String getStateShow()
	{ return stateShow; }
	
	
	/**
	 * <code>appendItem</code> appends the XML representation of the
	 * current packet data to the specified <code>StringBuffer</code>.
	 *
	 * @param retval The <code>StringBuffer</code> to append to
	 */
	public final void appendItem(StringBuffer retval)
	{
		retval.append("<presence");
		
		//append the base attributes from ContentPacket.java
		appendBaseAttribs(retval);
		retval.append(">");
		
		appendAnyError(retval);
		appendChild(retval,"status",status);
		appendChild(retval,"show",stateShow);
		
		if(getPriority() >= 0)
			retval.append("<priority>" + String.valueOf(getPriority()) + "</priority>");
		
		Enumeration extensions=Extensions();
		while (extensions.hasMoreElements())
		((Extension)extensions.nextElement()).appendItem(retval);
		
		retval.append("</presence>");
	}
}