/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import org.jabber.jabberbeans.Extension.*;
import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;

/**
 * This class is an abstraction of presence packets - both incoming and 
 * outgoing.
 *
 * Incoming presence packets are notifications on a resource - they are
 * sent by the remote client's server when that client has a status change.
 *
 * Outgoing presence packets are of two types. You send out your presence by
 * sending a packet to the server with your new status. The second type is a
 * subscribe request, where you request a remote user add you to their
 * presence notifications.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.12 $
 */
public class PresenceBuilder extends PacketBuilder implements Serializable
{
	/** status string. */
	private String status;

	/** priority - higher is better. */
	private int priority;

	/** stateshow - the icon 'state' of the user */
	private String stateShow;
	
	
	/** Creates a new <code>PresenceBuilder</code> instance. */
	public PresenceBuilder()
	{ reset(); }
	
	
	/**
	 * <code>Reset</code> is used to restore all values to their null/nothing
	 * values, so that the builder can be safely reused.
	 *
	 */
	public void reset()
	{
		resetBase();
		status = stateShow = null;
		priority = -66666;
	}
	
	
	/**
	 * Gets the status associated with this presence message, if any
	 *
	 * @return a <code>String</code> value, or null if no status
	 */
	public String getStatus()
	{ return status; }
	
	
	/**
	 * Sets the status associated with this presence message, if any
	 *
	 * @param status a <code>String</code> value, or null to clear
	 */
	public void setStatus(String status)
	{ this.status=status; }
	
	
	/**
	 * Get the priority level being set by this message
	 *
	 * @return an <code>int</code> value, with zero being the default
	 */
	public int getPriority()
	{ return priority; }
	
	
	/**
	 * Set the priority level being set by this message
	 *
	 * @param priority an <code>int</code> value, or zero to clear.
	 */
	public void setPriority(int priority)
	{ this.priority=priority; }
	
	
	/**
	 * Gets the state (icon) mentioned in this presence by name
	 *
	 * @return a <code>String</code> value, or null if none set
	 */
	public String getStateShow()
	{ return stateShow; }
	
	
	/**
	 * Sets the state (icon) to be mentioned in this presence by name
	 *
	 * @param state a <code>String</code> value, or null to clear
	 */
	public void setStateShow(String state)
	{ stateShow=state; }
	
	
	/**
	 * Generates a new object based in this builder's state
	 *
	 * @return a <code>Packet</code> value
	 * @exception java.lang.InstantiationException if not enough information/
	 * malformed information was passed.
	 */
	public Packet build() throws java.lang.InstantiationException
	{ return new Presence(this); }
}