/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.io.Serializable;
import java.io.IOException;
import java.util.*;
import org.jabber.jabberbeans.Extension.*;

/**
 * RosterBean is a bean to gain and utilize the roster information in jabber.
 * A 'Roster' is a list of users, a type of subscription list or address book,
 * if you will. By using a roster, you can get the status of other users 
 * (online/offline, etc), subscribe and unsubscribe users, as well as change
 * your own status.
 * <p>
 * This bean hooks into an IQBean, it will not work until it is given a
 * ConnectionBean to handle. RosterBean to ConnectionBean is a many to one
 * relation - one ConnectionBean can have many RosterBeans, but if software
 * wants to use more than a singular server connection, there is need for
 * multiple RosterBeans, one per ConnectionBean.
 * <p>
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.22 $
 */
public class RosterBean
    implements Serializable
{
    /** parent connection class, an IQBean */
    private IQBean connection;

    /** roster subscribed listener list */
    private Vector rosterListeners=new Vector();

    /** we use the roster extension object to store our current roster. */
    private Hashtable currentRoster=new Hashtable();
    
    /** an infoquery builder for sending roster change requests. Not created
     * until first use.
     */
    private InfoQueryBuilder IQBuilder=null;

    /** a roster extension builder for sending roster change requests. Not
     * created until first use.
     */
    private RosterBuilder rosterBuilder=null;

    /** a roster item builder for sending custom roster changes. Not created
     * until first use.
     */
    private RosterItemBuilder rosterItemBuilder=null;

    /** Internal object for processing the PacketListener events */
    private RosterPacketListener listener=null;

    /**
     * class <code>RosterPacketListener</code> filters out received packets
     * and outputs only those which contain roster information, through the
     * RosterListener vector.
     */
    class RosterPacketListener
        implements PacketListener
    {
        /**
         * <code>receivedPacket</code> of PacketListener handler, this filters
         * for jabber:iq:auth extensions, then broadcasts those events on a 
         * special RosterListener interface.
         *
         * @param p incoming <code>PacketEvent</code>
         */
        public void receivedPacket(PacketEvent p)
        {
	    if (!(p.getPacket() instanceof InfoQuery))
	     return;
            Enumeration e=((InfoQuery)p.getPacket()).Extensions();
            Extension ext;
            if ((e==null)||(!e.hasMoreElements())) 
                return;

            ext=(Extension)e.nextElement();

            if (!(ext instanceof Roster))
                    return;

            //TODO: add code to modify current roster image based on 
            //new data. Replacement is basically shown by if the type is not
            //'result' of the IQ
	    Roster rext= (Roster) ext;
	    
            if (((InfoQuery)p.getPacket()).getType().equals("result"))
            {
                //the results of a get will be the results of a full fetch
		Enumeration enum=rext.items();
		currentRoster.clear();
		while (enum.hasMoreElements())
		{
		    RosterItem ri=(RosterItem)enum.nextElement();
		    currentRoster.put(ri.getJID(),ri);
		}
                fireUserRosterReplaced(rext);
            }
            else
            {
		Enumeration enum=rext.items();
		while (enum.hasMoreElements())
		{
		    RosterItem ri=(RosterItem)enum.nextElement();
		    currentRoster.put(ri.getJID(),ri);
		}

                fireUserRosterChanged(rext);
            }
        }
        
        public void sentPacket(PacketEvent p)
        {
            //do nothing - no need to monitor outgoing, since the server sends
            //us back results
        }

	public void sendFailed(PacketEvent p)
	{
	    //do nothing here
	}

    }

    /**
     * Default do-nothing constructor.
     */
    public RosterBean()
    {}

    /**
     * <code>setIQBean</code> sets the IQBean 'parent' for this class, which
     * is used to send and receive events from the interface with the jabber
     * server.
     *
     * @param connection an <code>IQBean</code>
     */
    public void setIQBean(IQBean connection)
    {
        this.connection=connection;
        listener=new RosterPacketListener();
        connection.addPacketListener(listener);
    }

    /**
     * <code>getIQBean</code> returns the currently set parent bean, if any
     *
     * @return an <code>IQBean</code> value, or null if none has been set.
     */
    public IQBean getIQBean()
    { return connection; }
    
    /**
     * <code>refreshRoster</code> does a complete reload of the user roster.
     *
     * @exception InstantiationException should never be thrown (may be removed
     * after this is proven to be true)
     */
    public void refreshRoster()
        throws InstantiationException
    {
        //create builder if not used before.
        if (IQBuilder==null)
            IQBuilder=new InfoQueryBuilder();
        else
            //we have used  this builder before - make sure it is clean.
            IQBuilder.reset();
        IQBuilder.setType("get");

	if (rosterBuilder==null)
	    rosterBuilder=new RosterBuilder();
	else
	    rosterBuilder.reset();

        //this should never throw an exception
        IQBuilder.addExtension(rosterBuilder.build());
        
        //build and send out over the interface
        connection.send((InfoQuery)IQBuilder.build());
    }
        
    /**
     * <code>addRosterItem</code> adds one precreated roster item to the
     * roster, as stored on the server. A response will be sent by the server,
     * which will trigger a changed event on any registered RosterListener
     * interface.
     *
     * @param ri a <code>RosterItem</code> object to add
     * @exception InstantiationException if the build processes fail - this
     * may be removed after debugging.
     */
    public synchronized void addRosterItem(RosterItem ri)
        throws InstantiationException
    {
        //create builder if not used before.
        if (IQBuilder==null)
            IQBuilder=new InfoQueryBuilder();
        else
            //we have used  this builder before - make sure it is clean.
            IQBuilder.reset();
        IQBuilder.setType("set");
        
        if(rosterBuilder==null)
            rosterBuilder=new RosterBuilder();
            else
                rosterBuilder.reset();
        
        rosterBuilder.addRosterItem(ri);
        
        //this should never throw an exception, because we added the
        //roster item above first
        IQBuilder.addExtension(rosterBuilder.build());
        
        //this also should just work.
        connection.send((InfoQuery)IQBuilder.build());
    }

    /**
     * <code>delRosterItem</code> deletes a roster item. The rosterItem can
     * be taken directly from a valid Roster Extension object.
     *
     * The server will respond to this, at which time any registered
     * RosterListener interfaces will be notified via a 'changed' type event.
     *
     * @param ri a <code>RosterItem</code> to delete
     * @exception InstantiationException if there is an error building objects,
     * this may be removed later, after debugging.
     */
    public synchronized void delRosterItem(RosterItem ri)
        throws InstantiationException
    {   
        //create builder if not used before.
        if (IQBuilder==null)
            IQBuilder=new InfoQueryBuilder();
        else
            //we have used  this builder before - make sure it is clean.
            IQBuilder.reset();
        IQBuilder.setType("set");
        
        if(rosterBuilder==null)
            rosterBuilder=new RosterBuilder();
        else
            rosterBuilder.reset();

        if (rosterItemBuilder==null)
            rosterItemBuilder=new RosterItemBuilder();
        else
            //we have used this builder before - make sure it is clean.
            rosterItemBuilder.reset();

        // copy existing roster item
        rosterItemBuilder.copyItem(ri);

        //change subscription type to 'remove' to tell the server to delete the
        //item
        rosterItemBuilder.setSubscriptionType("remove");

        //this should never throw an exception, since we are based on
        //the original roster item.
        rosterBuilder.addRosterItem(rosterItemBuilder.build());
        
        //this should never throw an exception, because we added the
        //roster item above first
        IQBuilder.addExtension(rosterBuilder.build());

        //this also should just work.
        connection.send((InfoQuery)IQBuilder.build());
    }

    /**
     * <code>addRosterListener</code> registers a class supporting 
     * RosterListener to receive events from this bean.
     *
     * @param l a <code>RosterListener</code> to register
     */
    public synchronized void addRosterListener(RosterListener l)
    {
      if(!rosterListeners.contains(l))
				rosterListeners.addElement(l);   
    }

    /**
     * <code>delRosterListener</code> unregisters a RosterListener from
     * receiving events from this bean.
     *
     * @param l a <code>RosterListener</code> to unregister
     */
    public synchronized void delRosterListener(RosterListener l)
    { 
			if(rosterListeners.contains(l))
				rosterListeners.removeElement(l);
		}


    /**
     * returns the <code>entries</code> int the maintained roster cache.
     *
     * @return an <code>Enumeration</code> of RosterItems
     */
    public Enumeration entries()
    { return currentRoster.elements(); }

    /*
     * <code>fireUserRosterChanged</code> is a helper method to notify all
     * subscribed RosterListener objects that the roster has been altered.
     *
     * @param r an <code>Roster Extension</code> value holding a description
     * of changes made.
     */
    private void fireUserRosterChanged(Roster r)
    {
			try
			{
				Vector broadcast = (Vector)rosterListeners.clone();
        for(Enumeration e = broadcast.elements(); e.hasMoreElements();)
					((RosterListener)e.nextElement()).changedRoster(r);
			}
			catch(Exception e){e.printStackTrace();}
    }

    /*
     * <code>fireUserRosterReplaced</code> is a helper method to notify all
     * subscribed RosterListener objects that the roster has been completely
     * reloaded
     *
     * @param r an <code>Roster Extension</code> after reloading.
     */
    private void fireUserRosterReplaced(Roster r)
    {
			try
			{
				Vector broadcast = (Vector)rosterListeners.clone();
				for(Enumeration e = broadcast.elements(); e.hasMoreElements();)
					((RosterListener)e.nextElement()).replacedRoster(r);
			}
			catch(Exception e){e.printStackTrace();}
    }
}