/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import org.jabber.jabberbeans.util.JID;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * A <code>RosterItem</code> is a single user item in Jabber. A roster item
 * contains the following information:
 * <ul>
 * <li>JID: the Jabber Identifier of the subscribed resource. Usually people 
 * subscribe to a user and not to a resource.</li>
 *
 * <li>Subscription Type: the type of subscription between the user specified
 * and the actual client. Can be 'none', 'to' (we receive their presence), 
 * 'from' (they receive our presence) or 'both'. A fifth, non-visible option
 * is 'remove', which is sent to delete a message and returned to indicate an
 * item has been removed and should be deleted from the local cache.</li>
 *
 * <li>Ask State: the user has a state associated, which can be 'none' 
 * (default), 'subscribe', or 'unsubscribe'. This indicates there is a pending
 * request for subscription or unsubscription with the remote object.</li>
 *
 * <li>Name: a 'pretty' name, or nickname, for the user. This is what should
 * be displayed usually instead of a JID.</li>
 *
 * <li>Groups: one user can belong to many groups. Usually people will only put
 * a user in one group at a time, however. A list of groups can only be found
 * by getting the entire roster, and adding unique group names as the list is
 * parsed.</li>
 * </ul>
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.17 $
 *
 * @see Serializable
 */
public class RosterItem
    extends XMLData
{
    /** The JID as described above */
    private JID jid;

    /** The Subscription Type as described above */
    private String subscriptionType;

    /** The Asking State as described above */
    private String askStateType;

    /** The 'Friendly Name' as described above */
    private String friendlyName;

    /** A vector of groups associated with this item, as described above */
    private Vector groups=null;

    /**
     * Creates a new <code>RosterItem</code> instance based on the builder
     * being passed in
     *
     * @param builder a valid <code>RosterItemBuilder</code>
     * @exception InstantiationException if an error occurs
     */
    public RosterItem(RosterItemBuilder builder)
        throws InstantiationException
    {
        //get the JID
        jid=builder.getJID();
        if (jid==null) 
            throw new InstantiationException("No JID specified");

        //get the subscription type
        subscriptionType=builder.getSubscriptionType();
        if ((subscriptionType==null)||
            (subscriptionType.equals(""))||
            (subscriptionType.equals("none")))
            subscriptionType=null;
        else
        {
      	    if (!((subscriptionType.equals("to")) ||
                  (subscriptionType.equals("from")) ||
                  (subscriptionType.equals("both")) ||
		  (subscriptionType.equals("remove"))
                  ))
                throw new InstantiationException("Invalid subscription type");
	    subscriptionType.intern();
	}
           
        //get the in-process state type
        askStateType= builder.getWaitingStateType();
        if ((askStateType==null) ||
            (askStateType.equals("")) ||
	    (askStateType.equals("none")))
            askStateType=null;
        else 
	{
            if(!((askStateType.equals("subscribe")) ||
                 (askStateType.equals("unsubscribe"))
                 ))
                throw new InstantiationException("Invalid in-process request");
	    askStateType.intern();
	}

        //get the (locally set) friendly name
        friendlyName=builder.getFriendlyName();
        if ((friendlyName==null) || (friendlyName.equals(""))) 
            friendlyName=null;

	if (friendlyName!=null)
	    friendlyName.intern();
        
        groups=(Vector)builder.getGroups().clone();
    }
        
    /**
     * <code>getJID</code> gets the JID contained in this roster element
     *
     * @return a <code>JID</code> value
     */
    public JID getJID()
    { return jid; }

    /**
     * <code>getSubscriptionType</code> gets the subscription type of this 
     * roster element
     *
     * @return <code>String</code> holding subscription type, or null if none
     * specified.
     */
    public String getSubscriptionType()
    { return subscriptionType; }

    /**
     * <code>getWaitingStateType</code> returns a state that you are waiting
     * for, such as 'subscribe' (waiting for other user to approve your 
     * subscription). This value is not directly changable by clients.
     *
     * @return a <code>String</code> value, with null representing 'none'.
     */
    public String getWaitingStateType()
    { return askStateType; }

    /**
     * <code>getFriendlyName</code> returns the 'friendly name' associated with
     * this roster item
     *
     * @return a <code>String</code> value, or null if no friendly name was
     * specified
     */
    public String getFriendlyName()
    { return friendlyName; }

    /**
     * <code>enumerateGroups</code> enumerates the group(s) which the user
     * is entered. If no groups are specified or an 'empty' group is 
     * specified, the user is on the top level.
     *
     * @return an <code>Enumeration</code> value
     */
    public Enumeration enumerateGroups()
    { 
        if (groups==null)
            return null;
        return groups.elements();
    }

    /**
     * <code>appendItem</code> outputs an XML-style representation of the 
     * current roster item, appending it to a Stringbuffer.
     *
     * @return <code>StringBuffer</code> with XML
     */
    public void appendItem(StringBuffer sb)
    {
        sb.append("<item");

	appendAttrib(sb, "jid", getJID());
        appendAttrib(sb, "subscription", getSubscriptionType());
	appendAttrib(sb, "ask", getWaitingStateType());
	appendAttrib(sb, "name", getFriendlyName());
	
        Enumeration e = enumerateGroups();
        if (e==null)
        {
            sb.append("/>");
            return;
        }
        
        sb.append(">");
        while (e.hasMoreElements())
        {
            Object groupname=e.nextElement();
            if (groupname==null)
                sb.append("<group/>");
            else
		appendChild(sb,"group",groupname.toString());
        }
        sb.append("</item>");
    }
}
