/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import org.jabber.jabberbeans.util.JID;
import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;


/**
 * <code>RosterItemBuilder</code> is used in the creation of a roster item.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.11 $
 */
public class RosterItemBuilder
    implements Serializable
{
    /** The JID as described above */
    private JID jid;

    /** The Subscription Type as described above */
    private String subscriptionType;
 
    /** The Asking State as described above */
    private String askStateType;

    /** The 'Friendly Name' as described above */
    private String friendlyName;

    /** A vector of groups associated with this item, as described above */
    private Vector groups=null;

    /** Creates a new <code>RosterItemBuilder</code> instance. */
    public RosterItemBuilder()
    { reset(); }
    
    /** <code>reset</code> restores the default values so that the builder can
     * be reused.*/
    public void reset()
    {
        jid=null;
        subscriptionType=askStateType=friendlyName=null;
        groups=new Vector();
    }

    /** <code>getJID</code> returns the currently set JID.
     * @return a <code>JID</code>
     */
    public JID getJID()
    { return jid; }

    /**
     * <code>setJID</code> sets the current JID.
     *
     * @param jid a <code>JID</code> value
     */
    public void setJID(JID jid)
    { this.jid=jid; }

    /**
     * <code>getSubscriptionType</code> returns the specified subscription
     * type, if any
     *
     * @return a <code>String</code> value, or null if not specified.
     */
    public String getSubscriptionType()
    { return subscriptionType; }

    /**
     * <code>setSubscriptionType</code> sets the subscription type - note
     * that a client cannot alter this value through the roster interface.
     *
     * @param subscriptionType a <code>String</code> value, or null to clear
     */
    public void setSubscriptionType(String subscriptionType)
    { this.subscriptionType=subscriptionType; }

    /**
     * <code>getWaitingStateType</code> returns the waiting subscription
     * request, if any
     *
     * @return a <code>String</code> value, or null if none
     */
    public String getWaitingStateType()
    { return askStateType; }

    /**
     * <code>setWaitingStateType</code> sets the waiting subscription request.
     * note that clients cannot change this value through this interface.
     *
     * @param type a <code>String</code> value, or null to clear.
     */
    public void setWaitingStateType(String type)
    { askStateType=type; }

    /**
     * <code>getFriendlyName</code> returns the friendly name, if one exists
     *
     * @return a <code>String</code> value, or null if none
     */
    public String getFriendlyName()
    { return friendlyName; }

    /**
     * <code>setFriendlyName</code> sets the friendly name
     *
     * @param friendlyName a <code>String</code> value, or null to clear.
     */
    public void setFriendlyName(String friendlyName)
    { this.friendlyName=friendlyName; }

    /**
     * <code>getGroups</code> returns a vector of the groups currently
     * registered
     *
     * @return a <code>Vector</code> value
     */
    public Vector getGroups()
    { return groups; }

    /**
     * <code>addGroup</code> adds a group name (string) to the groups list
     * for this item.
     *
     * @param groupName a <code>String</code> value
     */
    public void addGroup(String groupName)
    { groups.addElement(groupName); }

    /**
     * <code>copyItem</code> copies information from an existing roster item.
     *
     * @param ri a <code>RosterItem</code> to copy.
     */
    public void copyItem(RosterItem ri)
    {
        setJID(ri.getJID());
        setSubscriptionType(ri.getSubscriptionType());
        setWaitingStateType(ri.getWaitingStateType());
        setFriendlyName(ri.getFriendlyName());
        Enumeration e= ri.enumerateGroups();
        groups=new Vector();

        //now step through and copy the groups.
        while (e.hasMoreElements())
            groups.addElement(e.nextElement());
    }
        
    /**
     * <code>build</code> creates a roster item based on this builder's state
     *
     * @return a <code>RosterItem</code> value
     * @exception InstantiationException if insufficient or malformed data
     * was specified.
     */
    public RosterItem build()
        throws InstantiationException
    { return new RosterItem(this); }
}
