/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans;

import java.io.Serializable;

/**
 * an XMLData object is the root class of a heirarchy of objects, all which
 * serialize themselves to XML data.
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.3 $
 */
public abstract class XMLData
    implements Serializable
{
    /**
     * <code>toString</code> is a serializer for the data contained in the
     * object to an equivalent snippet of XML stream.
     *
     * @return a <code>String</code> value which contains the XML 
     * representation of this object
     */
    public String toString()
    {
	StringBuffer retval=new StringBuffer();
	appendItem(retval);
	return new String(retval);
    }

    /**
     * <code>appendItem</code> appends the XML representation of the
     * current packet data to the specified <code>StringBuffer</code>.
     *
     * @param retval The <code>StringBuffer</code> to append to
     */
    public abstract void appendItem(StringBuffer retval);

    /**
     * <code>appendChild</code> adds the XML for a child 'element' to a
     * StringBuffer, as a helper to appendItem.
     *
     * @param retval a <code>StringBuffer</code> value
     * @param name a <code>String</code> value
     * @param value a <code>String</code> value
     *
     * @return a <code>bool</code> which is true if the element was generated.
     *         This is used for any shortcuts in the outputted XML
     */
    protected static final boolean appendChild(StringBuffer retval,
					       String name, 
					       String value)
    {
        if (value==null) 
            return false;
        retval.append( '<' );
        retval.append(name);
        retval.append( '>' );
        escapeString(retval, value);
        retval.append( "</" );
        retval.append(name);
        retval.append( '>' );
	return true;
    }

    /**
     * <code>appendChild</code> outputs an element if needed. If value=false
     * it does not output the tag.
     *
     * @param retval a <code>StringBuffer</code> value
     * @param name a <code>String</code> value
     * @param value a <code>boolean</code> value
     *
     * @return a <code>bool</code> which is true if the element was generated.
     *         This is used for any shortcuts in the outputted XML
     */
    protected static final boolean appendChild(StringBuffer retval,
					       String name, 
					       boolean value)
    {
        if (value==false)
            return false;
        retval.append( '<' );
        retval.append(name);
        retval.append( "/>" );
	return true;
    }

    /**
     * <code>appendAttrib</code> outputs an attribute if needed. If value=null
     * it will not output the attribute. Note that it outputs in the format:<p>
     * &qout;&nbsp;<i>attrib</i>=<i>value</i>&qout;, or it puts a proceeding
     * space in front of the attribute, and no trailing space. This is for
     * optimizing the XML generation.
     *
     * @param retval a <code>StringBuffer</code> value
     * @param name a <code>String</code> value
     * @param value a <code>boolean</code> value
     *
     * @return a <code>bool</code> which is true if the attribute was 
     * generated. This is used for any shortcuts in the outputted XML
     */
    protected static final boolean appendAttrib(StringBuffer retval,
						String name,
						String value)
    {
	if (value==null) 
	    return false;
	retval.append( ' ' );
	retval.append(name );
	retval.append("=\"");
	escapeString(retval,value);
	retval.append("\"");
	return true;
    }

    /**
     * <code>appendAttrib</code> outputs an attribute if needed. If value=null
     * it will not output the attribute. Note that it outputs in the format:<p>
     * &qout;&nbsp;<i>attrib</i>=<i>value</i>&qout;, or it puts a proceeding
     * space in front of the attribute, and no trailing space. This is for
     * optimizing the XML generation.
     *
     * @param retval a <code>StringBuffer</code> value
     * @param name a <code>String</code> value
     * @param value a <code>Object</code> value, which supports toString()
     *
     * @return a <code>bool</code> which is true if the attribute was 
     * generated. This is used for any shortcuts in the outputted XML
     */
    protected static final boolean appendAttrib(StringBuffer retval,
						String name,
						Object value)
    {
	if (value==null)
	    return false;
	return appendAttrib(retval,name,value.toString());
    }

    protected static final void escapeString(StringBuffer retval, String data)
    {
	int length=data.length();
	char c;
	for (int i=0;i<length;i++)
	{
	    switch(c=data.charAt(i))
	    {
	    case '&':
		retval.append("&amp;");
		break;
	    case '<':
		retval.append("&lt;");
		break;
	    case '>':
		retval.append("&gt;");
		break;
	    case '\'':
		retval.append("&apos;");
		break;
	    case '\"':
		retval.append("&quot;");
		break;
	    default:
		retval.append(c);
	    }
	}
    }
}
