/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.sax.Extension;

import org.jabber.jabberbeans.*;
import org.jabber.jabberbeans.sax.*;

import org.xml.sax.AttributeList;
import org.xml.sax.SAXException;

/**
 * Handler class to build DefaultExtension objects (objects without a
 * supplied custom handler). This serializes the data back out to
 * equivalent XML
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.15 $ 
 */
public class SerializingHandler
    extends SubHandler
{
    /** buffer for accumulated characters while parsing the 'packet' */
    private StringBuffer accumulated;

    /** slot for the base elemnt name */
    private String element;

    /** slot for storing the namespace of the object */
    private String xmlns;

    /**
     * Creates a new <code>SerializingHandler</code> instance.
     */
    public SerializingHandler()
    {
	super();
	accumulated=new StringBuffer();
    };

    /**
     * Gets called when the underlying engine decides to pass an entity and
     * all sub-entities off to your subhandler.<p>
     *
     * Upon seeing the element that this subhandler handles, we call this
     * constructor, passing in the attributes.
     *
     * @param attributes a value of type 'AttributeList'
     */
    public void startHandler(String name,AttributeList attributes)
	throws SAXException
    {
	accumulated=new StringBuffer();
	xmlns=attributes.getValue("xmlns");
	element=name;
	handleStartElement(name,attributes);
    };
    
    /**
     * This is an exact copy of the start element in the main handler.
     *
     * @param name string that holds the element name
     * @param attributes AttributeList of attributes going with this element
     * @exception SAXException thrown on error (unexpected element)
     */
    public void handleStartElement(String name, AttributeList attributes)
	throws SAXException
    {
	accumulated.append('<');
	accumulated.append(name);
	for (int i = 0; i<attributes.getLength();i++)
	{
	    accumulated.append(' ');
	    accumulated.append(attributes.getName(i));
	    accumulated.append('=');
	    accumulated.append('\"');
	    accumulated.append(attributes.getValue(i));
	    accumulated.append('\"');
	}
	accumulated.append('>');
    }
    
    /**
     * This is an exact copy of the end element in the main handler
     *
     * @param name string holding the element name
     * @exception SAXException thrown on error
     */
    public void handleEndElement(String name)
	throws SAXException
    {
	accumulated.append("</");
	accumulated.append(name);
	accumulated.append('>');
    }

    /**
     * This is an exact copy of the characters function in the main handler
     *
     * @param ch character string detected
     * @param start start position
     * @param length length of string
     * @exception SAXException thrown on error
     */
    public void characters(char[] ch, int start, int length)
	throws SAXException
    {
	accumulated.append(new String(ch,start,length));
    }
    
    /**
     * Stophandler is the same as end element, except that it is called saying
     * that the subhandler is no longer in scope.
     *
     * @param nil a value of type ''
     */
    public Object stopHandler(String name)
	throws SAXException
    {
	handleEndElement(name);
	try
	{
	    return new DefaultExtension(element,xmlns,new String(accumulated));
	}
	catch (InstantiationException e)
	    {
		e.fillInStackTrace();
		throw new SAXException(e);
	    }
    }

    /**
     * reset the handler to a default state, for possible reuse.
     */
    public void reset()
    {
	accumulated=new StringBuffer();
    }

    /**
     * received data handler from subobjects. Ignored since this
     * handler never creates child handlers
     *
     * @param subHandler child subhandler which is returning
     * @param data Object which the child is returning
     */
    public void receiveChildData(SubHandler subHandler, Object data)
    {}
}
