/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.sax;

import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.InputStream;
import org.xml.sax.*;

/** This class is responsible for translating xmlns="*" tags into appropriate
 * object builders for creating Extension-derived objects. 
 *
 * The objects returned from this factory are SAX handlers, which are then put
 * in charge of the stream from the server until the extension is officially 
 * 'over'
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: dwaite $
 * @version $Revision: 1.3 $ 
 */
public class HandlerFactory
{
    /** lookup list for tag -> HandlerEntry */
    private Hashtable hash;

    /** filename for the Extension properties file, which is used
     * to generate the HandlerEntry hash */
    private static final String FILENAME=
	"/Properties/Handlers.properties";

    /** default Handler for tags and extensions which do not have a custom
     * handler */
    private HandlerEntry defaultHandler=null;    

    /** 
     * constructor, loads property file and fills in hash table
     *
     * @throws FileNotFounntException if the properties file is not found
     * @throws IOException if there is another I/O problem getting the
     *         properties file
     */
    public HandlerFactory()
	throws java.io.FileNotFoundException, 
	       java.io.IOException
    {
	Properties properties=new Properties();

	//get properties file as located within any classpath
	InputStream is=getClass().getResourceAsStream(FILENAME);

	if (is==null) 
	    throw new java.io.FileNotFoundException
		("properties for Handler factory not found");
	properties.load(is);
	
	Enumeration e=properties.propertyNames();

	hash=new Hashtable();

	//scan through, looking for .ns or .element suffixes
	while (e.hasMoreElements())
	{
	    String element;
	    if ((element=(String)e.nextElement()).endsWith(".ns"))
	    {
		String shortenedElement=
		    element.substring(0,element.length()-3); //truncate .ns
		if (hash.get(shortenedElement)!=null)
		    throw new RuntimeException
			("Duplicate entries in file "+ FILENAME);
		
		String propertyValue=properties.getProperty(element);
		
		HandlerEntry he;
		hash.put(propertyValue,
			 he = new HandlerEntry
			     (shortenedElement,propertyValue,null));
	    }
	    else if (element.endsWith(".element"))
	    {
		String shortenedElement=
		    element.substring(0,element.length()-8);//truncate .element
		if (hash.get(shortenedElement)!=null)
		    throw new RuntimeException
			("Duplicate entries in file "+ FILENAME);
		
		String propertyValue=properties.getProperty(element);
		
		HandlerEntry he;
		hash.put(propertyValue,
			 he = new HandlerEntry
			     (shortenedElement,null,propertyValue));
	    }
	}
	
	//now fill in rest of data on all hashtable entries
	e=hash.elements();
	while (e.hasMoreElements())
	{
	    HandlerEntry entry = (HandlerEntry)e.nextElement();
	    entry.setDescription
		(properties.getProperty(entry.getPrefix()+".desc"));
	    entry.setBuilder
		(properties.getProperty(entry.getPrefix()+".builder"));
	    entry.setHandler
		(properties.getProperty(entry.getPrefix()+".handler"));			    
	    entry.setProduct
		(properties.getProperty(entry.getPrefix()+".product"));
	}

	//finally, load in default
	defaultHandler=
	    new HandlerEntry("xmlns.default","default",null);
	defaultHandler.setDescription
	    (properties.getProperty(defaultHandler.getPrefix()+".desc"));
	defaultHandler.setBuilder
	    (properties.getProperty(defaultHandler.getPrefix()+".builder"));
	defaultHandler.setHandler
	    (properties.getProperty(defaultHandler.getPrefix()+".handler"));			    
	defaultHandler.setProduct
	    (properties.getProperty(defaultHandler.getPrefix()+".product"));

	//done!
	e=null;
	properties=null;
    }

    /** locates and constructs the appropriate class handling the xmlns. If 
     * none is located, throws an exception. The default handler (i.e. one that
     * just represents the xmlns and internal data as XML) is not included 
     * here, if you would like to use that handler aquire it with locateDefault
     *
     * @param name element name for lookup
     * @param attributes attributes for lookup (scans for xmlns)
     *
     * @return SubHandler if an appropriate handler is found, otherwise null
     */
    public SubHandler getHandlerInstance(String name, AttributeList attributes)
    {
	HandlerEntry entry;
	    entry=locateEntry(name,attributes);
	try
	{
	    if (entry==null)
		entry=locateDefaultEntry();
	    return entry.getHandlerClass();
	}
	catch (Exception e)
	{ 
	    if (e instanceof RuntimeException) throw (RuntimeException)e;
	    if (entry==null)
		throw (RuntimeException)e;
	    System.err.println("error creating handler class: " +
			       entry.getHandler()); 
	    e.printStackTrace();
	}
	return null;
    }

    /**
     * <code>locateEntry</code> returns all information on the handler for a
     * particular xmlns, if a handler is present.
     *
     * @param XMLNS a <code>String</code> value
     * @return an <code>HandlerEntry</code> value, or null if none
     * available.
     */
    public HandlerEntry locateEntry(String XMLNS)
    {
	HandlerEntry he = (HandlerEntry)hash.get(XMLNS);
	String handlerName;
	
	if (he==null)
	{
	    if (hash.containsKey(XMLNS))
		throw new RuntimeException("Invalid hashtable");
	    else
		return null;
	}
	return he;
    }

    /**
     * locates the default handler, the global handler that just contains 
     * the XML data and does not parse it into a java object. Unhandled 
     * extensions are contained by these objects
     *
     * @return HandlerEntry of the default handler
     */
    public HandlerEntry locateDefaultEntry()
    {
	if (defaultHandler.getHandler()==null)
	    throw new RuntimeException("No default specified");
	return defaultHandler;
    }

    /**
     * <code>locateEntry</code> scans for an appropriate handler, if any.
     *
     * It searches based on the element information passed from SAX on
     * startElement event.
     *
     * @param name a <code>String</code> holding an element name
     * @param attributes an <code>AttributeList</code> holding attributes,
     *      including XML namespaces
     * @return a <code>HandlerEntry</code>, if an appropriate one
     *      exists.
     */
    public HandlerEntry locateEntry(String name, AttributeList attributes)
    {
	//first, check for the base type. For instance, if there is a
	//handler for 'query', it is assumed this knows how to create all
	//objects for that element name, whether or not they have a namespace.

	HandlerEntry he = (HandlerEntry)hash.get(name);
	if (he!=null) return he;

	//if there is an xmlns, do a lookup based on that now
	String xmlns=attributes.getValue("xmlns");
	if (xmlns==null) return null;

	//there was an XMLNS, so we look it up
	he=(HandlerEntry)hash.get(xmlns);

	return he;
    }
}

