/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * dwaite 20001007
 * Changed reliance on ProtocolHandler to be a reliance on ConnectionBean.
 * This will probably change in the near future so that it instead receives
 * a reference to an accessor class. 
 */

package org.jabber.jabberbeans.sax;

import org.xml.sax.Parser;
import org.xml.sax.HandlerBase;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.DocumentHandler;
import org.jabber.jabberbeans.ConnectionBean;
import org.jabber.jabberbeans.Packet;

import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Creates a thread and handles the input part of an XMLStream via
 * a SAX parser
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.29 $ 
 */
public final class InputStreamHandler
    extends Thread
{
    /** InputStream which we are parsing XML from */
    private InputStream in=null;

    /** parent ConnectionBean object, for passing messages back */
    private ConnectionBean.InputStreamInterface isi = null;

    /** flag to indicate parsing in progress */
    private boolean parsingDone;

    /** Holds the name of the SAX-enabled parser. */
    private String parsername;

    private static final String CHANGE_PARSER_TOO_LATE =
	"You must set the parser before connecting.";

    /** Default parser (AElfred) */
    private static final String DEFAULT_PARSER = 
	"com.microstar.xml.SAXDriver";

    /**
     * Creates a new <code>InputStreamHandler</code> instance.
     *
     * @param in an <code>InputStream</code> for the server connection
     */
    public InputStreamHandler(ConnectionBean.InputStreamInterface isi)
    {
	this.isi = isi;
	parsingDone=false;
	parsername = DEFAULT_PARSER;
    }

    public void setInputStream(InputStream in)
    {
	this.in = in;
    }

    /**
     * Main thread body, creates the parser, associates the XMLStream
     * DocumentHandler, and starts everything running.
     */
    public final void run()
    {
	Parser parser;
	XMLStreamDocumentHandler docHandler;
	
	if (in == null)
	    throw new RuntimeException
		("attempting to start InputStreamHandler without input to " +
		 "monitor");
	try
	{
	    parser = (Parser)Class.forName(parsername).newInstance();
	}
	catch (Exception e)
	{
	    throw new RuntimeException("InputStreamHandler:"+e.toString());
	}

	docHandler=new XMLStreamDocumentHandler();
	docHandler.setPacketHandler(isi);

	((SubHandler)docHandler).setParser(parser);

	/* commented out - needed for xerces 
	 * parser.setReaderFactory(new StreamingCharFactory());
	 */
        parser.setDocumentHandler(docHandler);

	try
	{
	    final InputSource is=new InputSource(new
		InputStreamReader(in,"UTF8"));

	    parser.parse(is);
	}
	catch (IOException e)
	{
	    if (parsingDone == true)
		return;

	    isi.unexpectedThreadDeath(e);
	}
	catch (SAXException e)
	{
	    if (parsingDone == true)
		return;
	    isi.unexpectedThreadDeath(e);
	}

	parsingDone=true;	
    }

    /**
     * <code>received</code> is a method for other objects to broadcast
     * packets recognised on the inputStream up to the ConnectionBean
     *
     * @param p a <code>Packet</code> which was just received and
     * recognised.
     */
    public void received(Packet p)
    {
	isi.received(p);
    }

    public void shutdown()
    {
	parsingDone=true;
    }

    public void setParser(String parsername)
    {
	if (in != null)
	    throw new RuntimeException(CHANGE_PARSER_TOO_LATE);
	this.parsername = parsername;
    }

    public String getParser()
    {
	return parsername;
    }
}
