/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * dwaite 20001007
 * Changed reliance on ProtocolHandler to be a reliance on ConnectionBean.
 * This will probably change in the near future so that it instead receives
 * a reference to an accessor class. 
 */

package org.jabber.jabberbeans.sax;

import org.jabber.jabberbeans.Packet;
import org.jabber.jabberbeans.ConnectionBean;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Threaded queue for outgoing data, so that packets send asynchronously
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.16 $ 
 */
public final class OutputStreamHandler
    extends Thread
{
    /** output stream to the server */
    private OutputStream out;

    /** parent connection, used to send out events when a packet has been
     * successfully sent */
    private ConnectionBean.OutputStreamInterface osi;

    /** list out queued packets for output to the server. */
    private final Vector output = new Vector(10);

    /** boolean used to quietly shut down the thread */
    private boolean keepRunning = true;

    /**
     * Creates a new <code>OutputStreamHandler</code> instance.
     *
     * @param out an <code>OutputStream</code> to the server
     */
    public OutputStreamHandler(ConnectionBean.OutputStreamInterface osi)
    {
	this.osi=osi;
    }
    
    public void setOutputStream(OutputStream out)
    {
	this.out = out;
    }
			       
    /**
     * Adds an element to the queue for sending.
     *
     * @param p Packet to send
     */
    public final void send(Packet p)
    {
        synchronized(output)
        {
	    if (!keepRunning)
	    {
		osi.sendFailed(p);
		return;
	    }

            output.addElement(p);
	    output.notify();
        }
    }

    
    /**
     * <code>shutdown</code> is used to notify the thread to shutdown (such as
     * in the case of a client-requested disconnect).
     */
    public void shutdown()
    {
	keepRunning=false;
	synchronized(output)
	{
	    if (output.isEmpty())
	    {
		//we are probably waiting on connections
		output.notify();
	    }
	}
	try
	{
	    this.join();
	}
	catch (InterruptedException e)
	{
	    //do nothing
	}
    }

    public void handleThreadDeath(Exception ex,Packet p)
    {
	synchronized(output)
	{
	    Enumeration e = output.elements();

	    if (p != null)
		osi.sendFailed(p);

	    while (e.hasMoreElements())
	    {
		osi.sendFailed((Packet)e.nextElement());
	    }
	}
	osi.unexpectedThreadDeath(ex);
    }

    public void handleThreadDeath(Exception ex)
    {
	handleThreadDeath(ex, null);
    }

    /**
     * Thread which loops sending packets to the server, and sending out
     * messages when the packets have been sent out
     */
    public final void run()
    {
	Packet p;

	if (out == null)
	    throw new RuntimeException
		("starting output thread without any IO set up to use");

	while ( (keepRunning) ||
		(!output.isEmpty()) )
	{
	    // we synchronize on output, along with the 'send' method.
	    // if there is nothing to send, we will wait, and the send()
	    // method will notify us when data needs to be sent out.
	    synchronized(output) 
	    {
		if (output.isEmpty())
		{
		    p = null;
		}
		else
		{
		    p = (Packet)output.firstElement();
		    output.removeElement(p);
		}
                		
		if ((p == null) &&(keepRunning))
		{
		    try
		    {
			output.wait();
		    }
		    catch(InterruptedException e)
		    {
			e.printStackTrace();
			handleThreadDeath(e);
			return;
		    }
		}
	    }    

	    // we have a packet, we need to send it.
	    if (p!=null)
	    {
		try
		{
		    //convert packet to a UTF8 bytestream
		    out.write(p.toString().getBytes("UTF8"));
		    out.flush();
		}
		catch(UnsupportedEncodingException e)
		{
		    //JRE error - ALL should support UTF8
		    throw new Error
			("No UTF-8 character set support");
		}
		catch(IOException e)
		{
		    //we have an IO Exception - we call a handler method
		    //to indicate that we are dying, then exit the thread
		    // System.err.println("error sending: " + p.toString());
		    // e.printStackTrace();

		    //clean up local 
		    handleThreadDeath(e, p);
		    return;
		}
		// go to the connection bean interface to send out a success
		// full sent packet event this is sent on this output thread
		osi.sent(p);
	    }
	}

	try
	{
	    out.write("</stream:stream>".getBytes("UTF8"));
	    out.flush();
	}
	catch(Exception e)
	{}
    }
}
