/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.sax;

import org.jabber.jabberbeans.*;
import org.jabber.jabberbeans.Extension.*;
import org.jabber.jabberbeans.util.JID;

import org.xml.sax.AttributeList;
import org.xml.sax.SAXException;

/**
 * PacketSubHandler, which provides a framework to subclass for handling
 * base packet types.
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.23 $ 
 */
abstract public class PacketSubHandler
    extends SubHandler
{
    /** Builder class for packet types */
    protected PacketBuilder builder;

    /** Buffer for holding characters between elements. The only reason
     * a stringbuffer is used is that most parsers will divide character()
     * events by entities.
     *
     * i.e. "I can't make it tonite 'ok?'" would result in:
     * "I can" , "'" , "t make it tonight " , "'" , "ok?" , "'"
     */
    protected StringBuffer elementChars;
    
    /**
     * Construct a new PacketSubHandler class.Derived in subclasses.
     */
    public PacketSubHandler()
    {
	super();
	elementChars=null;
    }

    /**
     * This is an exact copy of the start element in the main handler.
     *
     * @param name string that holds the element name
     * @param attributes AttributeList of attributes going with this element
     * @exception SAXException thrown on error (unexpected element)
     */
    protected final void handleStartElement(String name, 
					    AttributeList attributes)
	throws SAXException
    {	
	// since this is only one element deep, we throw an exception
	// if we get another element while handling one (unless we have a
	// subhandler handling it)
	
	if (packetStartElement(name, attributes)) return;
	
	if ("error".equals(name))
	{
	    builder.setErrorCode(attributes.getValue("code"));
	    elementChars=new StringBuffer();
	    return;
	}
	
	// if this is has a handler, we switch off to it.
	HandlerFactory hf=getHandlerFactory();
	SubHandler newHandler=hf.getHandlerInstance(name,attributes);
	if (newHandler!=null)
	{
	    setChildSubHandler(newHandler,name,attributes);
	    return;
	}
	
	//unknown element type - so we ignore it
    }

    /**
     * <code>packetStartElement</code> is a method for handling additional 
     * element types not defined in the base packet type. It is addElement
     * after being filtered.
     *
     * @return a <code>boolean</code> value of false if the tag should be
     * referred to the extension factory. Basically a 'handled' bit.
     *
     * @exception SAXException if an error occurs
     */
    protected boolean packetStartElement(String name, AttributeList attributes)
	throws SAXException
    {return false;}

    /**
     * <code>startHandler</code> is called on the first element handled by
     * the class.
     *
     * This implementation sets up the builder, clears the element character
     * cache, and parsers attributes to pull in the base attribute data.
     *
     * @param name element name
     * @param attributes list of attributes
     */
    protected final void startHandler(String name, AttributeList attributes)
	throws SAXException
    {
	//first, reset the builder so we don't get any odd data carried over.
	builder.reset();
	builder.setIdentifier("none");
	elementChars=new StringBuffer();
	// now we step through the attributes - both pulling out data and
	// stuffing it into the messageBuilder, and verifying that there
	// aren't any unsupported attributes passed (like an xmlns)
	if (attributes.getValue("to")!=null)
	    builder.setToAddress
		(JID.fromString(attributes.getValue("to")));

	if (attributes.getValue("from")!=null)
	    builder.setFromAddress
		(JID.fromString(attributes.getValue("from")));

	if (attributes.getValue("id")!=null)
	    builder.setIdentifier(attributes.getValue("id"));

	if (attributes.getValue("type")!=null)
	    builder.setType(attributes.getValue("type"));

	packetStartHandler(name,attributes);
    }

    /**
     * <code>packetStartHandler</code> is overloaded by subclasses to initialize
     * additional attributes.
     *
     * @param name a <code>String</code> value
     * @param attributes an <code>AttributeList</code> value
     */
    protected void packetStartHandler(String name, AttributeList attributes)
    {    }
    /**
     * Report data characters seen by parser. This implementation appends
     * the data to the end of a StringBuffer
     *
     * @param ch character array
     * @param start offset into character array
     * @param length lenght of characters to pull from array
     */
    public final void characters(char[] ch,int start,int length)
	throws SAXException
    {	
	//for all other types, just accumulate the characters and put into
	//a string
	elementChars.append(ch,start,length);
    }

    /**
     * This is an exact copy of the end element in the main handler
     *
     * @param name string holding the element name
     * @exception SAXException thrown on error
     */
    protected final void handleEndElement(String name)
	throws SAXException
    {
	if (name.equals("error")) // = error
	    builder.setErrorText(new String(elementChars));

	packetEndElement(name);
    }

    /**
     * <code>handleEndElement</code> is a wrapper to let a class derived from
     * this class handle custom element types.
     *
     * @param name a <code>String</code> value holding the element name
     * @exception SAXException if an error occurs
     */
    protected void packetEndElement(String name)
	throws SAXException
    {}

    /**
     * called as the handler is losing scope, to return any data it has
     * created based on the XML it has been fed.
     *
     * @param name element name
     * @return Object with whatever relevant data. It is assumed that the
     *         parent will know what subclass the child data is in
     * @throws SAXException in the event of XML parsing errors, or 
     *         errors detected in the building process for creating the
     *         child data.
     */
    protected final Object stopHandler(String name)
	throws SAXException
    {
	try
	    {
		return builder.build();
	    }
	catch (InstantiationException e)
	    {
		//TODO: need better exception handling.
		throw new SAXException
		    ("PkSH: invalid XML/invalid build:" +e.toString());
	    }
    }

    /**
     * called when a child object returns data (via its stopHandler() method)
     *
     * @param subhandler SubHandler which is returning data
     * @param data Object being returned (parent is assumed to know how to
     *        subclass because it created the child)
     */
    protected final void receiveChildData(SubHandler subhandler,Object data)
    {
	builder.addExtension((Extension)data);
    }
}
