/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.sax;

import org.xml.sax.Parser;
import org.xml.sax.AttributeList;
import org.xml.sax.SAXException;
import org.xml.sax.DocumentHandler;
import org.xml.sax.Locator;

/**
 * Subhandlers are registered with the main document handler to handle all 
 * data within an element with a specific name. 
 * <p>
 * The basic idea is that a subhandler is built 'around' an element, so that
 * handing a particular type of packet in jabber is the responsibility of
 * different objects. For instance, PresenceHandler, MessageHandler, IQHandler.
 * <p>
 * The handler is constructed with the attribute list of the element, and is
 * stopped (via stopHandler) when the element is closed. In between these
 * two events, we get all the subelements and characters in the stream.
 * <p>
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.19 $ 
 */
public abstract class SubHandler
    implements DocumentHandler
{
    /** internal counter for doing start/endHandler methods. */
    private byte subCount;

    /** SAX parser being used, passed from parent */
    private Parser parser;

    /** parent SubHandler, which we return to after endHandler */
    private SubHandler parent;

    /** handlerFactory, passed from parent */
    private HandlerFactory handlerFactory;    

    /**
     * Creates a new <code>SubHandler</code> instance.
     */
    public SubHandler()
    {
	subCount=0;
	parser=null;
	parent=null;
	handlerFactory=null;
    }

    ////////////////////////////////////////////////////////////////////////
    // 
    // Following are the SAX methods which we always ignore. These are
    // here so we can properly implement DocumentHandler
    //
    ////////////////////////////////////////////////////////////////////////

    /** do-nothing implementation */
    public final void startDocument()
	throws SAXException
    {}

    /** do-nothing implementation */
    public final void endDocument()
	throws SAXException
    {}

    /** do-nothing implementation */
    public final void processingInstruction(String target, String data)
	throws SAXException
    {}
   
    /** do-nothing implementation */
    public final void setDocumentLocator(Locator locator)
    {}

    ////////////////////////////////////////////////////////////////////////
    //
    // Following are SAX methods which we "do" utilize.
    //
    ////////////////////////////////////////////////////////////////////////

    /**
     * This is an exact copy of the start element in the main handler.
     *
     * @param name string that holds the element name
     * @param attributes AttributeList of attributes going with this element
     * @exception SAXException thrown on error (unexpected element)
     */
    public final void startElement(String name, AttributeList attributes)
	throws SAXException
    { 
	subCount++;
	handleStartElement(name,attributes); 
    }

    /**
     * This is an exact copy of the end element in the main handler
     *
     * @param name string holding the element name
     * @exception SAXException thrown on error
     */
    public final void endElement(String name)
	throws SAXException
    {	
	if (subCount!=0)
	    {
		subCount--;
		handleEndElement(name); 
		return;
	    }
	//else we are ending this handler
	Object b=stopHandler(name);

	parser.setDocumentHandler(parent);
	parent.subCount--;
	parent.receiveChildData(this, b);
    }

    /**
     * This is an exact copy of the characters function in the main handler
     *
     * @param ch character string detected
     * @param start start position
     * @param length length of string
     * @exception SAXException thrown on error
     */
    public void characters(char[] ch, int start, int length)
	throws SAXException
    {}

    public void ignorableWhitespace(char[] ch, int start, int length)
	throws SAXException
    { characters(ch,start,length); }

    ////////////////////////////////////////////////////////////////////////
    //
    // Following are additional methods outside of the SAX DocumentHandler
    // interface which we provide for a consistant depth-first traversal
    // through the XML document.
    //
    ////////////////////////////////////////////////////////////////////////

    /**
     * Gets called when the underlying engine decides to pass an entity and
     * all sub-entities off to your subhandler.<p>
     *
     * Upon seeing the element that this subhandler handles, we call this
     * constructor, passing in the attributes.
     *
     * @param name name of the element which we are handling.
     * @param attributes list of attributes on this element
     */
    protected void startHandler(String name,AttributeList attributes)
	throws SAXException
    {}
        
    /**
     * <code>handleStartElement</code> is overloaded by the new class to
     * provide logic to handle the element code.
     *
     * @param name a <code>String</code> value
     * @param attributes an <code>AttributeList</code> value
     * @exception SAXException if an error occurs
     */
    protected void handleStartElement(String name, 
				      AttributeList attributes)
	throws SAXException
    {}
    
    /**
     * <code>handleEndElement</code> is overloaded by the new class to
     * provide logic to handle element code.
     *
     * @param name a <code>String</code> value
     * @exception SAXException if an error occurs
     */
    protected void handleEndElement(String name)
	throws SAXException
    {}
    
    /**
     * Stophandler is the same as end element, except that it is called saying
     * that the subhandler is no longer in scope.
     *
     * @param Object a value being returned to the parent - the parent is 
     * meant to interpret this result.
     */
    protected Object stopHandler(String name)
	throws SAXException
    {return null;}

    /**
     * <code>receiveChildData</code> is called when a child handler exits,
     * returning control to this code. The now-defunct handler along with the
     * data object are both returned.
     *
     * @param subHandler a <code>SubHandler</code> value
     * @param o an <code>Object</code> value
     * @throws SAXException if there is an error in the returned data
     */
    protected void receiveChildData(SubHandler subHandler,Object o)
	throws SAXException
    {}

    /**
     * <code>setParser</code> is used to set the SAX parser from the parent
     *
     * @param newParser a <code>Parser</code> value
     */
    public final void setParser(Parser newParser)
    { parser=newParser; }

    /**
     * <code>setParent</code> is used to set the parent object (from 
     * the parent)
     *
     * @param handler <code>SubHandler</code> which is this object's parent
     */
    public final void setParent(SubHandler handler)
    { parent=handler; }

    /**
     * <code>setHandlerFactory</code> is used to set the
     * HandlerFactory used to look up elements and namespace
     * extension handlers. This is set from the parent
     *
     * @param ehf <code>HandlerFactory</code>
     */
    public final void setHandlerFactory(HandlerFactory hf)
    { handlerFactory=hf; }


    /**
     * <code>getHandlerFactory</code> is used to get the
     * HandlerFactory used to look up elements and namespace
     * extension handlers. 
     *
     * @return <code>HandlerFactory</code>
     */
    public final HandlerFactory getHandlerFactory()
    { return handlerFactory; }

    /**
     * <code>getSubCount</code> returns the element level (the number of tags
     * deep the parser is in within the SubHandler, not globally)
     *
     * @return a <code>byte</code> value
     */
    protected final byte getSubCount()
    { return subCount; }

    /**
     * <code>setChildSubHandler</code> is called to hook up a new subHandler
     * to take over parsing the data stream, for instance in the case of
     * a new Extension handler.
     *
     * @param subHandler a <code>SubHandler</code> value
     * @param name a <code>String</code> value
     * @param attributes an <code>AttributeList</code> value
     * @exception SAXException if an error occurs
     */
    protected final void setChildSubHandler(SubHandler subHandler,
					    String name,
					    AttributeList attributes)
	throws SAXException
    {
	parser.setDocumentHandler(subHandler);
	subHandler.setParser(parser);
	subHandler.setParent(this);
	subHandler.setHandlerFactory(handlerFactory);
	subHandler.startHandler(name, attributes);
    }

    /**
     * <code>graftSubHandler</code> is ssimilar to setChildSubHandler,
     * except in that we are not specifying a handler for child tags, but 
     * actually one for the current sub-level. 
     *
     * The startHandler will be passed no parameters - do not try to
     * use them.
     *
     * When the subhandler finishes, you can write a method to receive
     * the child data, then call endElement yourself (with an appropriate
     * name for your own code) to exit the calling handler.
     *
     * @param subHandler a <code>SubHandler</code> value
     * @param name a <code>String</code> value
     * @param attributes an <code>AttributeList</code> value
     * @exception SAXException if an error occurs
     */
    protected final void graftSubHandler(SubHandler subHandler,
					    String name,
					    AttributeList attributes)
	throws SAXException
    {
	setChildSubHandler(subHandler,null,null);
	subHandler.startElement(name,attributes);
    }

    /**
     * <code>graftParent</code> is yet another ugly hack in the graft
     * series, which allows for XMLStreamHeader to be interpreted as
     * a packet instead of a new level. Without incrementing the parent's
     * count, the whole thing will go to the wrong level once the
     * parent exits, basically crashing the parser tree. Ugh.
     *
     * TODO: if XMLStreamDocumentHandler either parsed the headers itself
     * or was the parent of the current XMLStreamDocumentHandler, this
     * ugly hackedness would go away.
     *
     * @param name Name as would be passed to endElement if this was
     *             the end of a packet (same as the name attrib in
     *             a corresponding startElement)
     */
    protected final void graftParent(String name)
	throws SAXException
    {
	parent.subCount++;
	endElement(name);
    }	
}
