/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.sax.serverside;

import org.jabber.jabberbeans.sax.SubHandler;
import org.jabber.jabberbeans.serverside.LogBuilder;
import org.jabber.jabberbeans.util.JID;
import org.xml.sax.AttributeList;
import org.xml.sax.SAXException;

/**
 * This is the SubHandler for "<message>" objects. It uses a MessageBuilder
 * object to parse and create Message objects from the incoming XML, and
 * notifies the main bean (MessengerBean) when either a new message arrives,
 * an error arrives on an attempted sent message, or an error arrives in the 
 * server XML.
 * 
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: dwaite $
 * @version $Revision: 1.2 $ 
 */
public final class LogHandler 
    extends SubHandler
{
    /** builder for the log messages */
    protected LogBuilder builder;

    /** element character cache (for log messages) */
    protected StringBuffer elementChars;
    
    /** valid attributes */
    protected static final String baseAttribTypes[]=
    {
	"from",
	"type",
	"id"
    };


    /**
     * Creates a new <code>LogHandler</code> instance.
     *
     */
    public LogHandler()
    {
	super();
	elementChars=null;
	builder=new LogBuilder();
    }

    protected final void startHandler(String name, AttributeList attributes)
	throws SAXException
    {
	//first, reset the builder so we don't get any odd data carried over.
	builder.reset();
	builder.setIdentifier("none");
	elementChars=new StringBuffer();
	// now we step through the attributes - both pulling out data and
	// stuffing it into the messageBuilder, and verifying that there
	// aren't any unsupported attributes passed (like an xmlns)
	for (int i =0; i<attributes.getLength(); i++)
	{
	    boolean recognized=false;

	    for (int j =0; j<baseAttribTypes.length; j++)
	    {
		if (baseAttribTypes[j].equals(attributes.getName(i)))
		{
		    switch (j)
		    {
		    case 0: //from
			builder.setFromAddress
			    (JID.fromString(attributes.getValue(i)));
			break;

		    case 1: //type
			builder.setType(attributes.getValue(i));
			break;
		    case 2: //id
			builder.setIdentifier(attributes.getValue(i));
		    default:
		    }
		    recognized=true;
		    break;
		}
	    }
	}
    }

    public final void characters(char[] ch,int start,int length)
	throws SAXException
    {	
	//for all other types, just accumulate the characters and put into
	//a string
	elementChars.append(ch,start,length);
    }

    /**
     * This is an exact copy of the end element in the main handler
     *
     * @param name string holding the element name
     * @exception SAXException thrown on error
     */
    protected final void handleEndElement(String name)
	throws SAXException
    { }

    protected final Object stopHandler(String name)
	throws SAXException
    {
	// element holds content (log data), so we copy that to the builder
	builder.setContent(new String(elementChars));
	return builder.build();
    }
}
