/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.serverside;

import org.jabber.jabberbeans.*;
import org.jabber.jabberbeans.Extension.*;
import org.jabber.jabberbeans.util.JID;
import java.util.Vector;
import java.io.Serializable;

/**
 * A <code>Log</code> object represents a single Log packet, which is 
 * used to send debug/error information between the server components and any
 * data store.<p>
 * <i>e.g.:</i><br>
 * &lt;log type="notice|warn|error" from="service|'_internal'"&gt;<br>
 * &nbsp;&nbsp;content
 * &lt;log&gt;<p>
 * 
 * Note that there is no 'error' capacity, no 'to' attribute, and no xmlns 
 * extensions. IDs are pointless.
 *
 * @see ContentPacket
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.2 $
 */
public class Log
    extends XMLData
    implements Packet
{
    /** The address that this packet is coming from - your address (or blank)
     * for outgoing packets, the originator's address for incoming packets. */
    protected JID fromAddress;

    /** An identifier that is echoed in data replies/errors of this packet.
     * Identifiers are meant to be unique per XML stream, and not duplicated.*/
    protected String identifier;

    /** A packet subtype. For instance, a message can be a normal message, a
     * message in a one-on-one chat, a groupchat message or a headline. */
    protected String type;

    /** A <code>String</code> holding the loggable message. */
    protected String content;
    /**
     * Creates a new <code>Log</code> instance. 
     *
     * @param builder an <code>LogBuilder</code> holding appropriate
     * values
     */
    public Log(LogBuilder builder)
    {
        fromAddress=builder.getFromAddress();
        type=builder.getType();
	if (type!=null) type.intern();

        identifier=builder.getIdentifier();
	content=builder.getContent();
        /* note: we always use an identifier- if one is not specified by
         * the client, we insert our own. */
        if (identifier==null)
            identifier=IdentifierCounter.getNewIdentifier();
        else
            if (identifier.equals("none"))
                //they specified they wanted no identifier - set to null.
                identifier=null;
	
	if (identifier!=null) identifier.intern();
    }

    /**
     * <code>getFromAddress</code> returns the address of the sender of this
     * message. Can be blank (null).
     *
     * @return <code>JID</code> this packet originated from.
     */
    public JID getFromAddress()
    {
        return fromAddress;
    }

    /**
     * <code>getIdentifier</code> returns a string which uniquely identifies 
     * this packet in the XML stream. Any reply (such as an IQ type=result or
     * an error message) will carry this same identifier. Identifiers are not
     * meant to be duplicated in a stream.
     *
     * @return a <code>String</code> value
     */
    public String getIdentifier()
    {
        return identifier;
    }

    /**
     * <code>getType</code> returns the subtype of this packet. Its context
     * and correctness is determined by the final packet type (message, iq, etc)
     * according to the jabber protocol specifications on <a 
     * href="http://protocol.jabber.org">protocol.jabber.org</a>.
     *
     * @return <code>String</code> holding the type
     */
    public String getType()
    {
        return type;
    }

    /**
     * <code>appendItem</code> appends the XML representation of the
     * current packet data to the specified <code>StringBuffer</code>.
     */
    public void appendItem(StringBuffer retval)
    {
        retval.append("<log");
        //append contentPacket attributes.

	appendAttrib(retval,"from",fromAddress);
	appendAttrib(retval,"id",identifier);
	appendAttrib(retval,"type",type);

        retval.append(">"); 
	retval.append(content);
        retval.append("</log>");
    }
}
