/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */


package org.jabber.jabberbeans.util;


import java.io.Serializable;


/**
 * A <code>JID</code> is a jabber identifier, signifying a resource in the
 * Jabber protocol.
 * Format:
 * <i>[User@]server[/resource]</i>
 * <ul>
 * <li>User specifies the username you are talking to. Without a username
 * the data is being directed specifically at the server. This is checked by
 * the server, which will give an error of the username is invalid.</li>
 * <li>Server specifies the server that we are sending to. This is not checked
 * except by the jabber server, which will give an error if the server is 
 * invalid.</li>
 * <li>Resource is a unique identifier which distinguishes between multiple
 * sessions of a user. For some this has special meaning, such as 
 * distinguishing individual users for a groupchat. Without a resource 
 * specified, the message goes to a default resource.</li>
 * </ul> 
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: shawnxb $
 * @version $Revision: 1.8 $
 */


public class JID implements Serializable, Comparable
{
	/** optional username portion of the JID  */
	private String username;

	/** required server name portion of the JID. (Note in some server setups 
	 * this will not be a resolvable DNS name */
	private String server;

	/** optional resource being indexed by this JID */
	private String resource;

	/*
	 * Creates a new <code>JID</code> instance, with only a server being 
	 * specified.
	 *
	 * @param server <code>String</code> naming the server.
	 * @throws RuntimeException if the server field is null, as that
	 * would be an invalid JID.
	 */
	public JID(String server)
	{
		if(server == null || "".equals(server))
				throw new RuntimeException("illegal null server value");

		this.server = server;
		this.username = null;
		this.resource = null;
	}

	/**
	 * Creates a new <code>JID</code> instance, based on the server and user
	 * and resource strings. the Username and Resource can optionally be set
	 * to null.
	 *
	 * @param username <code>String</code> value holding the username, or null
	 * @param server <code>String</code> value holding the server name
	 * @param resource <code>String</code> value holding the resource, or null
	 *
	 * @throws RuntimeException if the server field is null (as that would be
	 * an invalid JID)
	 */
	public JID(String username, String server, String resource)
	{
		this(server);
		this.username=username;
		this.resource=resource;

		if("".equals(username))
			this.username=null;

		if("".equals(resource))
			this.resource=null;
	}

	/**
	 * <code>fromString</code> is a static method that creates a new JID based
	 * on an input string in URI format. 
	 *
	 * @param input <code>String</code> value containing a URI JID
	 * @return a <code>JID</code> value if we can parse this into a JID, null
	 * otherwise
	 */
	public static JID fromString(String input)
	{
			String res, user, svr;

			//first we strip off any leading "jabber:"
			if (input.regionMatches(true,           /* ignore case  */
															0,              /* offset       */
															"jabber:",      /* other string */
															0,              /* 2nd offset   */
															7))             /* length       */
					input=input.substring(7);

			//second, we look for an '@'
			int location = input.indexOf("@");
			if (location==-1)
				user=null;
			else
			{
				user=input.substring(0,location);
				//everything after the @ becomes the new string
				input=input.substring(location+1);
			}

			//third, we look for a resource
			location= input.indexOf("/");
			if(location==-1)
			{
				//not found, we don't have a resource and all of input 
				//(as it stands) is the servername
				res=null;
				svr=input;
			}
			else
			{
				//found, server is the left part, resource is the right part
				svr = input.substring(0,location);
				res = input.substring(location+1);
			}
			if(svr==null || "".equals(svr))
				return null;
			return new JID(user,svr,res);
	}

	/**
	 * Returns the username of the JID if one was specified, othewise null
	 *
	 * @return <code>String</code> holding the username, or null
	 */
	public String getUsername()
	{ return username;}

	/**
	 * Returns the server name of the JID
	 *
	 * @return <code>String</code> holding the server name
	 */
	public String getServer()
	{ return server; }

	/**
	 * Returns the resource if one was specified, otherwise null
	 *
	 * @return <code>String</code> holding the resource name, or null
	 */
	public String getResource()
	{ return resource; }

	/**
	 * Creates a string representing the JID. Note to get a URI, you
	 * need to prefix this with "jabber:". In communications with the server, 
	 * the jabber: tag is not required.
	 *
	 * @return <code>String</code> value representing the JID.
	 */
	public String toString()
	{
		StringBuffer out = new StringBuffer();
		if(username != null)
		{
			out.append(username);
			out.append("@");
		}
		out.append(server);
		if(resource != null)
		{
			out.append("/");
			out.append(resource);
		}

		return new String(out);
	}

	/*
	 * Creates a 'user friendly' string, with parameters for things like logins
	 * stripped off. It is preferable in some cases to return this instead of
	 * a full JID to a user information screen, so people don't freak out that
	 * their password or other information is being specified.
	 *
	 * @return a <code>String</code> value
	 * @deprecated use {@link #toSimpleString()} instead, will
   * be removed before 1.0
	 */
	public String toPrettyString()
	{
		String s = this.toString();
		int location = s.indexOf('?');
		if(location == -1)
			return s;
		else
			return s.substring(0,location);
	}

	/*
	 * <code>toSimpleString</code returns a string of the current jid
	 * sans(minus) the resource
	 *
	 * @return a <code>String</code> value
	 */
	public String toSimpleString()
	{
		StringBuffer jid = new StringBuffer();

		if(username != null)
			jid.append(username + "@");
		jid .append(server);

		return(jid.toString());
	}

	/*
	 * <code>equals</code> returns true if these objects are equivalent.
	 * It takes into account that usernames and servers are case-insensitive,
	 * and that resources are case sensitive. It also takes into account that
	 * usernames and resources are optional, but requires them to be specified
	 * in both or none of the two JIDs being compared.
	 *
	 * @param u a <code>JID</code> we are comparing to this one
	 * @return a <code>boolean</code> that is true when the JIDs are equivalent
	 */
	public boolean equals(JID u)
	{return normalize().equals(u.normalize());}
	
	
	/*
	 * <code>equals</code> returns true if these objects are equivalent.
	 * It takes into account that usernames and servers are case-insensitive,
	 * and that resources are case sensitive. It also takes into account that
	 * usernames and resources are optional, but requires them to be specified
	 * in both or none of the two JIDs being compared.
	 *
	 * @param obj a <code>Object</code> we are comparing to this one
	 * @return a <code>boolean</code> that is true when the objects are equivalent
	 */
	public boolean equals(Object obj)   
	{
		if (!(obj instanceof JID))
			 return false;
		
		return equals((JID)obj);
	}

	
	/*
	 * <code>compareTo</code> returns true if these objects are equivalent.
	 * It takes into account that usernames and servers are case-insensitive,
	 * and that resources are case sensitive. It also takes into account that
	 * usernames and resources are optional, but requires them to be specified
	 * in both or none of the two JIDs being compared.
	 *
	 * @param obj a <code>Object</code> we are comparing to this one
	 * @return a <code>int</code> that determines lexographical significance
	 */
	public int compareTo(Object o)
	{
		if(!(o instanceof JID))
			return -2;
		
		return normalize().compareTo(((JID)o).normalize());
	}
	
	
	/*
	 * <code>compare</code> returns true if these objects are equivalent.
	 * It takes into account that usernames and servers are case-insensitive,
	 * and that resources are case sensitive. It also takes into account that
	 * usernames and resources are optional, but requires them to be specified
	 * in both or none of the two JIDs being compared.
	 *
	 * @param obj a <code>Object</code> we are comparing to this one
	 * @return a <code>int</code> that determines lexographical significance
	 */
	public int compare(Object o)
	{return compareTo(o);}
	
	
	/*
	 * <code>compare</code> returns true if these objects are equivalent.
	 * It takes into account that usernames and servers are case-insensitive,
	 * and that resources are case sensitive. It also takes into account that
	 * usernames and resources are optional, but requires them to be specified
	 * in both or none of the two JIDs being compared.
	 *
	 * @param obj a <code>Object</code> we are comparing this
	 * @param obj a <code>Object</code> to this
	 * @return a <code>int</code> that determines lexographical significance
	 */
	public static int compare(Object o, Object o2)
	{
		if(!(o instanceof JID) || !(o2 instanceof JID))
			return -2;
		
		return ((JID)o).normalize().compareTo(((JID)o2).normalize());
	}
	
	
	/*
	 * <code>hashCode</code> returns the hash code of the jid in lowercase
	 *
	 * @return a <code>int</code> that is the hash code
	 */
	public int hashCode()
	{return normalize().hashCode();}
	
	
	/*
	 * <code>normalize</code> returns a normalized string from the jid taken
	 * This means that the username and server have been lowercased and the
	 * resource remains the same
	 *
	 * @return a <code>String</code> that is normalized
	 */
	public static String normalize(JID j)
	{
		StringBuffer buffer = new StringBuffer();

		if(j.getUsername() != null)
			buffer.append(j.getUsername().toLowerCase() + "@");
		if(j.getServer() != null)
			buffer.append(j.getServer().toLowerCase());
		if(j.getResource() != null)
		buffer.append("/" + j.getResource());
		
		return buffer.toString();
	}
	
	
	/*
	 * <code>normalize</code> returns a normalized string,
	 * This means that the username and server have been lowercased and the
	 * resource remains the same
	 *
	 * @return a <code>String</code> that is normalized
	 */
	public String normalize()
	{
		StringBuffer buffer = new StringBuffer();

		if(getUsername() != null)
			buffer.append(getUsername().toLowerCase() + "@");
		if(getServer() != null)
			buffer.append(getServer().toLowerCase());
		if(getResource() != null)
		buffer.append("/" + getResource());
		
		return buffer.toString();
	}
}