/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 * dwaite 20001007
 * Changed reliance on ProtocolHandler to be a reliance on ConnectionBean.
 * This will probably change in the near future so that it instead receives
 * a reference to an accessor class. 
 */

package org.jabber.jabberbeans.util;


import org.jabber.jabberbeans.*;


public class KeepAlive extends Thread
{
	/** The <code>ConnectionBean</code> which we are keeping alive. */
	private ConnectionBean cb;

	/** timeout in milliseconds, after which a keepalive packet (space)
	 * will be sent. */
	private int timeout;

	/*
	 * <code>boolean</code> indicating that the thread is still in
	 * a running state (used to shut it down) 
	 */
	private boolean bRunning;

	/* 
	 * <code>boolean</code> indicating that the connection is down to
	 * the thread, so that it does not keep sending data out to the
	 * ether. 
	 */
	private boolean bConnected;

	/**
	 * KeepAlivePacket is rather simple - instead of being XML, it is 
	 * just a space. You can send KeepAlivePackets at a regular interval 
	 * to make sure a connection doesn't expire. This feature is totally 
	 * optional.
	 */
	public class KeepAlivePacket extends XMLData implements Packet
	{
		/**
		 * toString for this class is a serializer for the data
		 * contained in the object to an equivalent snippet of XML 
		 * stream.
		 *
		 * @return a <code>String</code> value which contains the XML 
		 * representation of this object
		 */
		public String toString()
		{return " ";}

		/**
		 * <code>appendItem</code> adds the XML for a child 'element' 
		 * to a StringBuffer
		 *
		 * @param retval a <code>StringBuffer</code> value
		 */
		public void appendItem(StringBuffer retval)
		{retval.append(' ');}
	}

	public class KeepAlivePacketListener implements PacketListener
	{
		public void sentPacket(PacketEvent pe)
		{KeepAlive.this.interrupt();}

		public void receivedPacket(PacketEvent pe)
		{KeepAlive.this.interrupt();}

		//condition handled elsewhere in ConnectionListener,
		//do nothing here.
		public void sendFailed(PacketEvent pe){}
	}

	public class KeepAliveConnectionListener extends ConnectionAdapter
	{
		public void connected(ConnectionEvent ce)
		{
			synchronized(KeepAlive.this)
			{
				//sync to prevent wait/notify race in KeepAlive.start()
				bConnected = true;
				KeepAlive.this.notify();
			}
		}

		public void disconnected(ConnectionEvent ce)
		{
			synchronized(KeepAlive.this)
			{
				//sync to prevent wait/notify race in KeepAlive.start()
				bConnected = false;
				KeepAlive.this.notify();
			}
		}
	}

	/**
	 * Creates a new <code>KeepAlive</code> instance.
	 *
	 * @param cb a <code>ConnectionBean</code> value
	 * @param timeout timer in milliseconds after last traffic to send
	 *                keepalive.
	 */
	public KeepAlive(ConnectionBean cb, int timeout)
	{
		bRunning = true;
		this.cb = cb;
		this.timeout = timeout;
		
		cb.addPacketListener(new KeepAlivePacketListener());
		
		synchronized(this)
		{
			// update with current state - synchronized to block
			// the ConnectionListener to prevent race between setting the
			// state.
			bConnected = (cb.getConnectionState() == ConnectionEvent.STATE_CONNECTED);
		}
	
		cb.addConnectionListener(new KeepAliveConnectionListener());
	
		KeepAlive.this.start();
	}

	public synchronized void shutdown()
	{
		bConnected = false;
		bRunning = false;
		KeepAlive.this.notify();
	}

	public void run()
	{
		KeepAlivePacket packet = new KeepAlivePacket();
		
		while (bRunning)
		{
	    while (bConnected)
	    {
				synchronized(this)
				{
					try
					{KeepAlive.this.wait(timeout);}
					catch (InterruptedException e)
					{continue;}
					catch(Exception e)
					{e.printStackTrace(); KeepAlive.this.shutdown();}

					cb.send(packet);
				}
	    }
	    synchronized(this)
	    {
				//we recheck connected here, silly enough, because it would
				//otherwise be possible that from the end of the while loop
				//to the time we get the syncronized lock, we could
				//reconnect - causing us to miss the notify and sleep for a
				//long, long time.
				if (bRunning && !bConnected)
				{
					try{KeepAlive.this.wait(0);}
					catch(InterruptedException e){}
				}
	    }
		}
	}
}