/*
 *   License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 *   Copyrights
 *
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1999-2000 David Waite 
 *
 *   Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 *   Changes
 *
 */

package org.jabber.jabberbeans.util;

import java.io.Serializable;
import org.jabber.jabberbeans.*;

/** 
 * This is an interface that extends a PacketListener, so that it will filter 
 * out messages that are not the response being looked for. This is an adapter.
 * <p>
 * The main use for this class is to 'wait' on the main thread until the
 * response for an IQ packet comes back. 
 * <p>
 *
 * @author  David Waite <a href="mailto:dwaite@jabber.com">
 *                      <i>&lt;dwaite@jabber.com&gt;</i></a>
 * @author  $Author: mass $
 * @version $Revision: 1.6 $
 */
public class SyncPacketListener 
    implements PacketListener, Serializable
{
    /** Parent ConnectionBean */
    private ConnectionBean connectionbean;

    /** 
     * internal packet value, used for reporting a Packet type to watch for,
     * or as a placeholder for returning a packet after a incoming match
     * is found.
     */
    private Packet waitPacket;

    /** 
    * internal placeholder value for a Class type representing a packet
    * type to watch out for.
    */
    private Class waitClass;

    /**
     * type indicator for a single listener to be able to listen to and 
     * respond to different methods of use.
     */
    private int waitType;

    private static final char WAITUNKNOWN=0;
    private static final char WAITFORID=1;
    private static final char WAITFORPACKETTYPE=2;

    /** boolean set to indicate to the main thread that the condition
     * was met (as opposed to just timing out */
    private boolean waitSuccessful;

    /**
     * Creates a new <code>IQPacketListener</code> instance.
     *
     * @param cbean parent <code>ConnectionBean</code>
     */
    public SyncPacketListener(ConnectionBean cbean)
    {
        connectionbean=cbean;
	reset();
    }
    
    /**
     * reset restores the object to a known state, it should always be called
     * if the object is reused.
     */
    public void reset()
    {
	waitType=WAITUNKNOWN;
	waitPacket=null;
	waitClass=null;
	waitSuccessful=false;
        connectionbean.addPacketListener(this);
    }

    /**
     * sends a packet, then waits for a response, blocking indefinately until
     * the response comes over the wire, or there is an exception
     *
     * @param p a ContentPacket which is sent. The identifier field is used
     *          for matching the response, so a ContentPacket is required.
     * @return matched <code>Packet</code> if successful
     */
    public Packet sendAndWait(ContentPacket p)
	throws InterruptedException
    {
	return sendAndWait(p,0);
    }

    /**
     * sends a packet, then waits for a response, blocking until the
     * response comes over the wire, there is a timeout, or there is an 
     * exception
     *
     * @param p a ContentPacket which is sent. The identifier field is used
     *          for matching the response, so a ContentPacket is required.
     * @return matched <code>Packet</code> if successful, otherwise null
     */
    public synchronized Packet sendAndWait(ContentPacket p, int timeout)
	throws InterruptedException
    {
	if ((!(p instanceof ContentPacket)) ||
	    (((ContentPacket)p).getIdentifier()==null))
	    throw new RuntimeException
		("Error: cannot block on non-ID packets");

	connectionbean.send(p);
	waitPacket=p;

	waitType=WAITFORID;
	wait(timeout);
	return waitSuccessful?waitPacket:null;
    }

    /**
     * wait indefinately for another packet of the same type to come in.
     *
     * @param p packet which incoming packets will be compared
     * @return matched <code>Packet</code> if successful
     */
    public Packet waitForType(Packet p)
	throws InterruptedException
    {
	return waitForType(p,0);
    }

    /**
     * wait indefinately for a packet of type Class 'c' to come in.
     *
     * @param c class which incoming packets will be compared.
     * @return matched <code>Packet</code> if successful
     */
    public Packet waitForType(Class c)
	throws InterruptedException
    {
	return waitForType(c,0);
    }

    /**
     * wait for a specified period of time for a packet of the same type as
     * Packet p to come in
     *
     * @param p packet which incoming packets will be compared.
     * @param timeout maximum time to wait
     * @return matched <code>Packet</code> if successful, otherwise null
     */
    public synchronized Packet waitForType(Packet p,int timeout)
	throws InterruptedException
    {
	waitPacket=p;
	waitType=WAITFORPACKETTYPE;
	wait(timeout);
	return waitSuccessful?waitPacket:null;
    }
	
    /**
     * wait for a specified period of time for a packet of the same type as
     * Class 'c' to come in
     *
     * @param class class which incoming packets will be compared.
     * @param timeout maximum time to wait
     * @return matched <code>Packet</code> if successful, otherwise null
     */
    public synchronized Packet waitForType(Class c,int timeout)
	throws InterruptedException
    {
	waitClass=c;
	waitType=WAITFORPACKETTYPE;
	wait(timeout);
	return waitSuccessful?waitPacket:null;
    }

    /**
     * Monitor <code>sentPacket</code> listener method
     *
     * @param p corresponding <code>PacketEvent</code> value
     */
    public void sentPacket(PacketEvent p)
    { }

    /**
     * Monitor <code>receivedPacket</code> listener method. Will fire when a 
     * packet matching the sent packet's identifier is found
     *
     * @param p a <code>PacketEvent</code> value
     */
    public synchronized void receivedPacket(PacketEvent p)
    {
	if (p.getSource()!=connectionbean) return;

	//if we get a message while having an unknown waittype, we
	//eject - we shouldn't have a client waiting for anything
	if (waitType==WAITUNKNOWN) return; 

	if (waitType==WAITFORID)
	{
	    //won't work on non-ContentPacket's
	    if (!(p.getPacket() instanceof ContentPacket))
		return;

	    ContentPacket cp=(ContentPacket) p.getPacket();

	    //no use in looking here if there isn't an id
	    if (cp.getIdentifier()==null)
		return;

	    //if equivalent identifiers, notify to wake up main thread
	    if (cp.getIdentifier().equals
		(((ContentPacket) waitPacket).getIdentifier()))
	    {
		waitPacket=p.getPacket();
		waitSuccessful=true;
		waitType=WAITUNKNOWN;
		connectionbean.delPacketListener(this);
		this.notify();

	    }
	    return;
	}
	if (waitType==WAITFORPACKETTYPE)
	{
	    if (waitPacket!=null)
		waitClass=waitPacket.getClass();

	    if (waitClass.equals(p.getPacket().getClass()))
	    {
		waitPacket=p.getPacket();
		waitSuccessful=true;
		waitType=WAITUNKNOWN;
		connectionbean.delPacketListener(this);
		this.notify();
	    }
	    return;
	}
    }

    public synchronized void sendFailed(PacketEvent pe)
    {
	//no matter what, we unblock with a failure state.
	waitPacket=null;
	waitSuccessful=false;
	waitType=WAITUNKNOWN;
	connectionbean.delPacketListener(this);
	this.notify();
    }
}
